@@ -32,8 +32,6 @@ extern "C" {
  */
 /** @todo: Somehow supply type **/
 typedef enum rpmTag_e {
-    RPMTAG_NOT_FOUND		= -1,			/*!< Unknown tag */
-
     RPMTAG_HEADERIMAGE		= HEADER_IMAGE,		/*!< Current image. */
     RPMTAG_HEADERSIGNATURES	= HEADER_SIGNATURES,	/*!< Signatures. */
     RPMTAG_HEADERIMMUTABLE	= HEADER_IMMUTABLE,	/*!< Original image. */
@@ -381,10 +379,20 @@ typedef enum rpmTag_e {
     RPMTAG_TRANSLATIONURL	= 5100, /* s */
     RPMTAG_UPSTREAMRELEASES	= 5101, /* s */
     RPMTAG_SOURCELICENSE	= 5102, /* internal */
+    RPMTAG_PREUNTRANS		= 5103,	/* s */
+    RPMTAG_POSTUNTRANS		= 5104,	/* s */
+    RPMTAG_PREUNTRANSPROG	= 5105,	/* s[] */
+    RPMTAG_POSTUNTRANSPROG	= 5106,	/* s[] */
+    RPMTAG_PREUNTRANSFLAGS	= 5107, /* i */
+    RPMTAG_POSTUNTRANSFLAGS	= 5108, /* i */
+    RPMTAG_SYSUSERS		= 5109, /* s[] extension */
+    RPMTAG_BUILDSYSTEM		= 5110, /* internal */
+    RPMTAG_BUILDOPTION		= 5111, /* internal */
 
     RPMTAG_FIRSTFREE_TAG	/*!< internal */
 } rpmTag;
 
+#define RPMTAG_NOT_FOUND		((uint32_t)-1)
 #define	RPMTAG_EXTERNAL_TAG		1000000
 
 /** \ingroup rpmtag
@@ -445,7 +453,7 @@ typedef enum rpmSigTag_e {
 /** \ingroup header
  * The basic types of data in tags from headers.
  */
-typedef enum rpmTagType_e {
+enum rpmTagType_e {
 #define	RPM_MIN_TYPE		1
     RPM_NULL_TYPE		=  0,
     RPM_CHAR_TYPE		=  1,
@@ -460,7 +468,8 @@ typedef enum rpmTagType_e {
 #define	RPM_MAX_TYPE		9
 #define RPM_FORCEFREE_TYPE	0xff
 #define RPM_MASK_TYPE		0x0000ffff
-} rpmTagType;
+};
+typedef rpmFlags rpmTagType;
 
 /** \ingroup rpmtag
  * The classes of data in tags from headers.
@@ -523,7 +532,7 @@ rpmTagClass rpmTagGetClass(rpmTagVal tag);
 /** \ingroup rpmtag
  * Return tag value from name.
  * @param tagstr	name of tag
- * @return		tag value, -1 on not found
+ * @return		tag value, RPMTAG_NOT_FOUND on not found
  */
 rpmTagVal rpmTagGetValue(const char * tagstr);
 
