@@ -12,13 +12,13 @@
 #include <rpm/rpmfi.h>
 #include <rpm/rpmstring.h>
 
-#include "lib/rpmts_internal.h"
-#include "lib/rpmte_internal.h"
-#include "lib/rpmds_internal.h"
-#include "lib/rpmfi_internal.h" /* rpmfiles stuff for now */
-#include "lib/misc.h"
+#include "rpmts_internal.h"
+#include "rpmte_internal.h"
+#include "rpmds_internal.h"
+#include "rpmfi_internal.h" /* rpmfiles stuff for now */
+#include "misc.h"
 
-#include "lib/backend/dbiset.h"
+#include "backend/dbiset.h"
 
 #include "debug.h"
 
@@ -30,15 +30,11 @@ const char * const rpmEVR = VERSION;
 
 const int rpmFLAGS = RPMSENSE_EQUAL;
 
-#undef HASHTYPE
-#undef HTKEYTYPE
-#undef HTDATATYPE
-
 #define HASHTYPE depCache
 #define HTKEYTYPE const char *
 #define HTDATATYPE int
-#include "lib/rpmhash.H"
-#include "lib/rpmhash.C"
+#include "rpmhash.H"
+#include "rpmhash.C"
 #undef HASHTYPE
 #undef HTKEYTYPE
 #undef HTDATATYPE
@@ -62,8 +58,8 @@ const int rpmFLAGS = RPMSENSE_EQUAL;
 
 #define HASHTYPE depexistsHash
 #define HTKEYTYPE rpmsid
-#include "lib/rpmhash.H"
-#include "lib/rpmhash.C"
+#include "rpmhash.H"
+#include "rpmhash.C"
 #undef HASHTYPE
 #undef HTKEYTYPE
 
@@ -161,6 +157,19 @@ rpmdbMatchIterator rpmtsPrunedIterator(rpmts ts, rpmDbiTagVal tag,
     return mi;
 }
 
+rpmdbMatchIterator rpmtsTeIterator(rpmts ts, rpmte te, int prune)
+{
+    rpmdbMatchIterator mi = rpmtsPrunedIterator(ts, RPMDBI_NAME, rpmteN(te), prune);
+    rpmdbSetIteratorRE(mi, RPMTAG_EPOCH, RPMMIRE_STRCMP, rpmteE(te));
+    rpmdbSetIteratorRE(mi, RPMTAG_VERSION, RPMMIRE_STRCMP, rpmteV(te));
+    rpmdbSetIteratorRE(mi, RPMTAG_RELEASE, RPMMIRE_STRCMP, rpmteR(te));
+    if (rpmtsColor(ts)) {
+	rpmdbSetIteratorRE(mi, RPMTAG_ARCH, RPMMIRE_STRCMP, rpmteA(te));
+	rpmdbSetIteratorRE(mi, RPMTAG_OS, RPMMIRE_STRCMP, rpmteO(te));
+    }
+    return mi;
+}
+
 /**
  * Decides whether to skip a package upgrade/obsoletion on TE color.
  *
@@ -522,7 +531,7 @@ static int rpmdbProvides(rpmts ts, depCache dcache, rpmds dep, dbiIndexSet *matc
     Header h = NULL;
     int rc = 0;
     /* pretrans deps are provided by current packages, don't prune erasures */
-    int prune = (rpmdsFlags(dep) & RPMSENSE_PRETRANS) ? 0 : 1;
+    int prune = (rpmdsFlags(dep) & (RPMSENSE_PRETRANS|RPMSENSE_PREUNTRANS)) ? 0 : 1;
     unsigned int keyhash = 0;
 
     /* See if we already looked this up */
@@ -657,7 +666,7 @@ exit_rich:
     rpmdbProvides(ts, NULL, dep, &set1);
 
     /* Pretrans dependencies can't be satisfied by added packages. */
-    if (!(dsflags & RPMSENSE_PRETRANS)) {
+    if (!(dsflags & (RPMSENSE_PRETRANS|RPMSENSE_PREUNTRANS))) {
 	rpmte *matches = rpmalAllSatisfiesDepend(tsmem->addedPackages, dep);
 	if (matches) {
 	    for (rpmte *p = matches; *p; p++)
@@ -771,7 +780,7 @@ exitrich:
     }
 
     /* Pretrans dependencies can't be satisfied by added packages. */
-    if (!(dsflags & RPMSENSE_PRETRANS)) {
+    if (!(dsflags & (RPMSENSE_PRETRANS|RPMSENSE_PREUNTRANS))) {
 	rpmte *matches = rpmalAllSatisfiesDepend(tsmem->addedPackages, dep);
 	int match = matches && *matches;
 	_free(matches);
@@ -784,7 +793,7 @@ exitrich:
 	goto exit;
 
     /* Search for an unsatisfied dependency. */
-    if (adding && !retrying && !(dsflags & RPMSENSE_PRETRANS)) {
+    if (adding && !retrying && !(dsflags & (RPMSENSE_PRETRANS|RPMSENSE_PREUNTRANS))) {
 	int xx = rpmtsSolve(ts, dep);
 	if (xx == 0)
 	    goto exit;
@@ -843,7 +852,7 @@ static void checkInstDeps(rpmts ts, depCache dcache, rpmte te,
     if (depds)
 	dep = rpmdsN(depds);
     if (neg) {
-	ndep = rmalloc(strlen(dep) + 2);
+	ndep = (char *)xmalloc(strlen(dep) + 2);
 	ndep[0] = '!';
 	strcpy(ndep + 1, dep);
 	dep = ndep;
@@ -1019,7 +1028,7 @@ int rpmtsCheck(rpmts ts)
     confilehash = filedepHashCreate(257, sidHash, sidCmp, NULL, NULL);
     connothash = depexistsHashCreate(257, sidHash, sidCmp, NULL);
     connotfilehash = filedepHashCreate(257, sidHash, sidCmp, NULL, NULL);
-    addIndexToDepHashes(ts, RPMTAG_CONFLICTNAME, NULL, confilehash, connothash, connotfilehash);
+    addIndexToDepHashes(ts, RPMDBI_CONFLICTNAME, NULL, confilehash, connothash, connotfilehash);
     if (!filedepHashNumKeys(confilehash))
 	confilehash = filedepHashFree(confilehash);
     if (!depexistsHashNumKeys(connothash))
@@ -1031,7 +1040,7 @@ int rpmtsCheck(rpmts ts)
     reqfilehash = filedepHashCreate(8191, sidHash, sidCmp, NULL, NULL);
     reqnothash = depexistsHashCreate(257, sidHash, sidCmp, NULL);
     reqnotfilehash = filedepHashCreate(257, sidHash, sidCmp, NULL, NULL);
-    addIndexToDepHashes(ts, RPMTAG_REQUIRENAME, NULL, reqfilehash, reqnothash, reqnotfilehash);
+    addIndexToDepHashes(ts, RPMDBI_REQUIRENAME, NULL, reqfilehash, reqnothash, reqnotfilehash);
     if (!filedepHashNumKeys(reqfilehash))
 	reqfilehash = filedepHashFree(reqfilehash);
     if (!depexistsHashNumKeys(reqnothash))
