@@ -16,19 +16,20 @@
 #include <rpm/rpmlog.h>
 #include <rpm/rpmstring.h>
 #include <rpm/argv.h>
+#include <rpm/rpmbase64.h>
 
-#include "lib/fsm.h"		/* XXX CPIO_FOO/FSM_FOO constants */
-#include "lib/rpmchroot.h"
-#include "lib/rpmfi_internal.h" /* XXX replaced/states... */
-#include "lib/rpmte_internal.h"	/* XXX internal apis */
-#include "lib/rpmdb_internal.h" /* rpmdbAdd/Remove */
-#include "lib/rpmts_internal.h" /* rpmtsPlugins() etc */
-#include "lib/rpmds_internal.h" /* rpmdsFilterTi() */
-#include "lib/rpmscript.h"
-#include "lib/misc.h"
-#include "lib/rpmtriggers.h"
+#include "fsm.h"		/* XXX CPIO_FOO/FSM_FOO constants */
+#include "rpmchroot.h"
+#include "rpmfi_internal.h" /* XXX replaced/states... */
+#include "rpmte_internal.h"	/* XXX internal apis */
+#include "rpmdb_internal.h" /* rpmdbAdd/Remove */
+#include "rpmts_internal.h" /* rpmtsPlugins() etc */
+#include "rpmds_internal.h" /* rpmdsFilterTi() */
+#include "rpmscript.h"
+#include "misc.h"
+#include "rpmtriggers.h"
 
-#include "lib/rpmplugins.h"
+#include "rpmplugins.h"
 
 #include "debug.h"
 
@@ -81,7 +82,7 @@ static rpmRC markReplacedFiles(const rpmpsm psm)
     if (num == 0)
 	return RPMRC_OK;
 
-    offsets = xmalloc(num * sizeof(*offsets));
+    offsets = (unsigned int *)xmalloc(num * sizeof(*offsets));
     offsets[0] = 0;
     num = prev = 0;
     for (sfi = replaced; sfi; sfi=rpmfsNextReplaced(fs, sfi)) {
@@ -163,6 +164,7 @@ rpmRC rpmInstallSourcePackage(rpmts ts, FD_t fd,
     Header h = NULL;
     rpmpsm psm = NULL;
     rpmte te = NULL;
+    rpmPlugins origplugins = NULL;
     rpmRC rpmrc;
     int specix = -1;
 
@@ -217,6 +219,10 @@ rpmRC rpmInstallSourcePackage(rpmts ts, FD_t fd,
 	    rpmfsSetAction(fs, i, FA_CREATE);
     }
 
+    /* Don't run any plugins */
+    origplugins = ts->plugins;
+    ts->plugins = rpmpluginsNew(ts);
+
     psm = rpmpsmNew(ts, te, PKG_INSTALL);
 
     if (rpmpsmUnpack(psm) == RPMRC_OK)
@@ -224,6 +230,10 @@ rpmRC rpmInstallSourcePackage(rpmts ts, FD_t fd,
 
     rpmpsmFree(psm);
 
+    /* Restore plugins */
+    rpmpluginsFree(ts->plugins);
+    ts->plugins = origplugins;
+
 exit:
     if (rpmrc == RPMRC_OK && specix >= 0) {
 	if (cookie)
@@ -251,7 +261,8 @@ static rpmRC runInstScript(rpmpsm psm, rpmTagVal scriptTag)
 
     if (script) {
 	headerGet(h, RPMTAG_INSTPREFIXES, &pfx, HEADERGET_ALLOC|HEADERGET_ARGV);
-	rc = runScript(psm->ts, psm->te, h, pfx.data, script, psm->scriptArg, -1);
+	rc = runScript(psm->ts, psm->te, h, (ARGV_const_t)pfx.data,
+			script, psm->scriptArg, -1);
 	rpmtdFreeData(&pfx);
     }
 
@@ -261,6 +272,166 @@ static rpmRC runInstScript(rpmpsm psm, rpmTagVal scriptTag)
     return rc;
 }
 
+struct ARGVi_s {
+    ARGV_t argv;
+    int argc;
+    int ix;
+};
+
+static const char *nextarg(void *data)
+{
+    const char *l = NULL;
+    struct ARGVi_s *avi = (struct ARGVi_s *)data;
+    if (avi->ix < avi->argc) {
+	l = avi->argv[avi->ix];
+	avi->ix++;
+    }
+    return l;
+}
+
+/* Execute all systemd-sysusers separately for all sources */
+static rpmRC execSysusers(rpmpsm psm, Header h, const char *cmd,
+			  ARGV_t *sysusers, int nsysusers)
+{
+    rpmRC rc = RPMRC_OK;
+
+    for (int i = 0; i < nsysusers; i++) {
+	const char *path = sysusers[i][0];
+	rpmScript script = NULL;
+	ARGV_t argv = NULL;
+
+	argvAdd(&argv, cmd);
+	if (*path) {
+	    argvAdd(&argv, "--replace");
+	    argvAdd(&argv, path);
+	}
+	if (!rstreq(rpmtsRootDir(psm->ts), "/")) {
+	    argvAdd(&argv, "--root");
+	    argvAdd(&argv, rpmtsRootDir(psm->ts));
+	}
+	argvAdd(&argv, "-");
+
+	script = rpmScriptFromArgv(h, RPMTAG_SYSUSERS, argv, 0, 0);
+	if (script) {
+	    struct rpmtd_s pfx;
+	    struct ARGVi_s avi = {
+		.argv = sysusers[i],
+		.argc = argvCount(sysusers[i]),
+		.ix = 1,
+	    };
+
+	    rpmScriptSetNextFileFunc(script, nextarg, &avi);
+	    headerGet(h, RPMTAG_INSTPREFIXES, &pfx,
+			HEADERGET_ALLOC|HEADERGET_ARGV);
+	    rc = runScript(psm->ts, psm->te, h, (ARGV_const_t)pfx.data,
+			   script, psm->scriptArg, -1);
+	    rpmtdFreeData(&pfx);
+	}
+	rpmScriptFree(script);
+	argvFree(argv);
+
+	if (rc)
+	    break;
+    }
+    return rc;
+}
+
+/* Return providing file path for provides index px, "" on not found. */
+static char *findProviderFile(rpmfiles files, int px)
+{
+    int fc = rpmfilesFC(files);
+    char *fn = NULL;
+
+    for (int i = 0; i < fc; i++) {
+	const uint32_t *ddict = NULL;
+	uint32_t ndict = rpmfilesFDepends(files, i, &ddict);
+	for (int j = 0; j < ndict; j++) {
+	    unsigned int dict = ddict[j];
+	    unsigned int dt = (dict >> 24) & 0xff;
+	    unsigned int dx = dict & 0x00ffffff;
+	    if (dt == 'P' && dx == px) {
+		fn = rpmfilesFN(files, i);
+		goto exit;
+	    }
+	}
+    }
+
+exit:
+    return (fn != NULL) ? fn : xstrdup("");
+}
+
+static rpmRC runSysusers(rpmpsm psm)
+{
+    rpmRC rc = RPMRC_OK;
+    Header h = NULL;
+    rpmds provides = NULL;
+    ARGV_t *sysusers = NULL;
+    int nsysusers = 0;
+    int dx;
+    char *cmd = rpmExpand("%{?__systemd_sysusers}", NULL);
+
+    /* Skip all this if disabled */
+    if (*cmd == '\0')
+	goto exit;
+
+    h = rpmteHeader(psm->te);
+    provides = rpmdsNew(h, RPMTAG_PROVIDENAME, 0);
+
+    /*
+     * Decode sysusers provides from this header into per-source argv's, 
+     * providing file as the first element ("" for non-file based provides),
+     * followed by decoded sysusers lines.
+     */
+    while ((dx = rpmdsNext(provides)) >= 0) {
+	char *line = NULL;
+	if (!rpmdsIsSysuser(provides, &line))
+	    continue;
+	char *fn = NULL;
+	int px = -1;
+
+	if (sysusers == NULL)
+	    sysusers = (ARGV_t *)xcalloc(rpmdsCount(provides), sizeof(*sysusers));
+
+	/* Find the providing file (if any) */
+	fn = findProviderFile(psm->files, dx);
+
+	/* Do we already know this provider? */
+	for (int i = 0; i < nsysusers; i++) {
+	    if (rstreq(sysusers[i][0], fn)) {
+		px = i;
+		break;
+	    }
+	}
+
+	/* If not, allocate one */
+	if (px == -1) {
+	    px = nsysusers;
+	    argvAdd(&sysusers[px], fn);
+	    nsysusers++;
+	}
+
+	argvAdd(&sysusers[px], line);
+
+	free(fn);
+	free(line);
+    }
+
+    if (sysusers) {
+	rc = execSysusers(psm, h, cmd, sysusers, nsysusers);
+
+	for (int i = 0; i < nsysusers; i++)
+	    argvFree(sysusers[i]);
+	free(sysusers);
+    }
+
+exit:
+    rpmdsFree(provides);
+    headerFree(h);
+    free(cmd);
+
+    return rc;
+}
+
 /**
  * Execute triggers.
  * @todo Trigger on any provides, not just package NVR.
@@ -288,7 +459,6 @@ static rpmRC handleOneTrigger(rpmts ts, rpmte te, rpmsenseFlags sense,
 	return rc;
 
     headerGet(trigH, RPMTAG_INSTPREFIXES, &pfx, HEADERGET_ALLOC|HEADERGET_ARGV);
-    (void) rpmdsSetNoPromote(trigger, 1);
 
     while ((i = rpmdsNext(trigger)) >= 0) {
 	uint32_t tix;
@@ -314,7 +484,8 @@ static rpmRC handleOneTrigger(rpmts ts, rpmte te, rpmsenseFlags sense,
 		rpmScript script = rpmScriptFromTriggerTag(trigH,
 			     triggertag(sense), RPMSCRIPT_NORMALTRIGGER, tix);
 		arg1 += countCorrection;
-		rc = runScript(ts, te, trigH, pfx.data, script, arg1, arg2);
+		rc = runScript(ts, te, trigH, (ARGV_const_t)pfx.data,
+				script, arg1, arg2);
 		if (triggersAlreadyRun != NULL)
 		    triggersAlreadyRun[tix] = 1;
 
@@ -381,7 +552,7 @@ static rpmRC runTriggers(rpmpsm psm, rpmsenseFlags sense)
 static rpmRC runImmedTriggers(rpmpsm psm, rpmsenseFlags sense)
 {
     const rpmts ts = psm->ts;
-    unsigned char * triggersRun;
+    uint8_t * triggersRun;
     struct rpmtd_s tnames, tindexes;
     Header h = rpmteHeader(psm->te);
     int nerrors = 0;
@@ -391,10 +562,10 @@ static rpmRC runImmedTriggers(rpmpsm psm, rpmsenseFlags sense)
 	goto exit;
     }
 
-    triggersRun = xcalloc(rpmtdCount(&tindexes), sizeof(*triggersRun));
+    triggersRun = (uint8_t *)xcalloc(rpmtdCount(&tindexes), sizeof(*triggersRun));
     {	Header sourceH = NULL;
 	const char *trigName;
-    	rpm_count_t *triggerIndices = tindexes.data;
+	rpm_count_t *triggerIndices = (rpm_count_t *)tindexes.data;
 
 	while ((trigName = rpmtdNextString(&tnames))) {
 	    rpmdbMatchIterator mi;
@@ -436,9 +607,24 @@ static rpmpsm rpmpsmFree(rpmpsm psm)
     return NULL;
 }
 
+static int isUpdate(rpmts ts, rpmte te)
+{
+    rpmtsi pi = rpmtsiInit(ts);
+    rpmte p;
+    int update = 0;
+    while ((p = rpmtsiNext(pi, TR_REMOVED)) != NULL) {
+	if (rpmteDependsOn(p) == te) {
+	    update = 1;
+	    break;
+	}
+    }
+    rpmtsiFree(pi);
+    return update;
+}
+
 static rpmpsm rpmpsmNew(rpmts ts, rpmte te, pkgGoal goal)
 {
-    rpmpsm psm = xcalloc(1, sizeof(*psm));
+    rpmpsm psm = (rpmpsm)xcalloc(1, sizeof(*psm));
     psm->ts = rpmtsLink(ts);
     psm->files = rpmteFiles(te);
     psm->te = te; /* XXX rpmte not refcounted yet */
@@ -446,7 +632,8 @@ static rpmpsm rpmpsmNew(rpmts ts, rpmte te, pkgGoal goal)
 	/*
 	 * When we run scripts, we pass an argument which is the number of
 	 * versions of this package that will be installed when we are
-	 * finished.
+	 * finished. countCorrection is only relevant for triggers
+	 * executing under PKG_INSTALL and PKG_ERASE.
 	 */
 	int npkgs_installed = rpmdbCountPackages(rpmtsGetRdb(ts), rpmteN(te));
 	switch (goal) {
@@ -455,14 +642,25 @@ static rpmpsm rpmpsmNew(rpmts ts, rpmte te, pkgGoal goal)
 	    psm->scriptArg = npkgs_installed + 1;
 	    psm->countCorrection = 0;
 	    break;
+	case PKG_PREUNTRANS:
+	case PKG_TRANSFILETRIGGERUN:
+	    if (rpmteDependsOn(psm->te))
+		psm->scriptArg = npkgs_installed;
+	    else
+		psm->scriptArg = npkgs_installed - 1;
+	    psm->countCorrection = -1;
+	    break;
+	case PKG_POSTUNTRANS:
+	    psm->scriptArg = npkgs_installed;
+	    break;
 	case PKG_ERASE:
 	    psm->scriptArg = npkgs_installed - 1;
 	    psm->countCorrection = -1;
 	    break;
 	case PKG_VERIFY:
 	case PKG_POSTTRANS:
-	    psm->scriptArg = npkgs_installed;
-	    psm->countCorrection = 0;
+	case PKG_TRANSFILETRIGGERIN:
+	    psm->scriptArg = npkgs_installed + isUpdate(psm->ts, psm->te);
 	    break;
 	default:
 	    break;
@@ -488,7 +686,7 @@ static rpmpsm rpmpsmNew(rpmts ts, rpmte te, pkgGoal goal)
     return psm;
 }
 
-void rpmpsmNotify(rpmpsm psm, int what, rpm_loff_t amount)
+void rpmpsmNotify(rpmpsm psm, rpmCallbackType what, rpm_loff_t amount)
 {
     if (psm) {
 	int changed = 0;
@@ -516,16 +714,7 @@ void rpmpsmNotify(rpmpsm psm, int what, rpm_loff_t amount)
  */
 static void markReplacedInstance(rpmts ts, rpmte te)
 {
-    rpmdbMatchIterator mi = rpmtsPrunedIterator(ts, RPMDBI_NAME, rpmteN(te), 1);
-    rpmdbSetIteratorRE(mi, RPMTAG_EPOCH, RPMMIRE_STRCMP, rpmteE(te));
-    rpmdbSetIteratorRE(mi, RPMTAG_VERSION, RPMMIRE_STRCMP, rpmteV(te));
-    rpmdbSetIteratorRE(mi, RPMTAG_RELEASE, RPMMIRE_STRCMP, rpmteR(te));
-    /* XXX shouldn't we also do this on colorless transactions? */
-    if (rpmtsColor(ts)) {
-	rpmdbSetIteratorRE(mi, RPMTAG_ARCH, RPMMIRE_STRCMP, rpmteA(te));
-	rpmdbSetIteratorRE(mi, RPMTAG_OS, RPMMIRE_STRCMP, rpmteO(te));
-    }
-
+    rpmdbMatchIterator mi = rpmtsTeIterator(ts, te, 1);
     while (rpmdbNextIterator(mi) != NULL) {
 	rpmteSetDBInstance(te, rpmdbGetIteratorOffset(mi));
 	break;
@@ -536,7 +725,7 @@ static void markReplacedInstance(rpmts ts, rpmte te)
 static rpmRC dbAdd(rpmts ts, rpmte te)
 {
     Header h = rpmteHeader(te);
-    rpm_time_t installTime = (rpm_time_t) time(NULL);
+    rpm_time_t installTime = rpmtsGetTime(ts, 1);
     rpmfs fs = rpmteGetFileStates(te);
     rpm_count_t fc = rpmfsFC(fs);
     rpm_fstate_t * fileStates = rpmfsGetStates(fs);
@@ -656,6 +845,11 @@ static rpmRC rpmPackageInstall(rpmts ts, rpmpsm psm)
 	if (rpmtsFilterFlags(psm->ts) & RPMPROB_FILTER_REPLACEPKG)
 	    markReplacedInstance(ts, psm->te);
 
+	/* Create sysusers.d(5) users and groups provided by this package */
+	if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_NOSYSUSERS)) {
+	    rc = runSysusers(psm);
+	    if (rc) break;
+	}
 
 	if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_NOTRIGGERPREIN)) {
 	    /* Run triggers in other package(s) this package sets off. */
@@ -672,34 +866,37 @@ static rpmRC rpmPackageInstall(rpmts ts, rpmpsm psm)
 	    if (rc) break;
 	}
 
-	if ((rc = rpmChrootIn()) == 0) {
+	rc = rpmChrootIn() ? RPMRC_FAIL : RPMRC_OK;
+	if (rc == RPMRC_OK) {
 	    rc = rpmpsmUnpack(psm);
 	    rpmChrootOut();
 	}
 	if (rc) break;
 
-	/*
-	 * If this package has already been installed, remove it from
-	 * the database before adding the new one.
-	 */
-	if (rpmteDBInstance(psm->te)) {
-	    rc = dbRemove(ts, psm->te);
+	if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_NODB)) {
+	    /*
+	     * If this package has already been installed, remove it from
+	     * the database before adding the new one.
+	     */
+	    if (rpmteDBInstance(psm->te)) {
+		rc = dbRemove(ts, psm->te);
+		if (rc) break;
+	    }
+
+	    rc = dbAdd(ts, psm->te);
 	    if (rc) break;
 	}
 
-	rc = dbAdd(ts, psm->te);
-	if (rc) break;
-
 	if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_NOTRIGGERIN)) {
 	    /* Run upper file triggers i. e. with higher priorities */
 	    /* Run file triggers in other package(s) this package sets off. */
-	    rc = runFileTriggers(psm->ts, psm->te, RPMSENSE_TRIGGERIN,
+	    rc = runFileTriggers(psm->ts, psm->te, psm->scriptArg, RPMSENSE_TRIGGERIN,
 				RPMSCRIPT_FILETRIGGER, 1);
 	    if (rc) break;
 
 	    /* Run file triggers in this package other package(s) set off. */
-	    rc = runImmedFileTriggers(psm->ts, psm->te, RPMSENSE_TRIGGERIN,
-				    RPMSCRIPT_FILETRIGGER, 1);
+	    rc = runImmedFileTriggers(psm->ts, psm->te, psm->scriptArg,
+				    RPMSENSE_TRIGGERIN, RPMSCRIPT_FILETRIGGER, 1);
 	    if (rc) break;
 	}
 
@@ -719,13 +916,13 @@ static rpmRC rpmPackageInstall(rpmts ts, rpmpsm psm)
 
 	    /* Run lower file triggers i. e. with lower priorities */
 	    /* Run file triggers in other package(s) this package sets off. */
-	    rc = runFileTriggers(psm->ts, psm->te, RPMSENSE_TRIGGERIN,
+	    rc = runFileTriggers(psm->ts, psm->te, psm->scriptArg, RPMSENSE_TRIGGERIN,
 				RPMSCRIPT_FILETRIGGER, 2);
 	    if (rc) break;
 
 	    /* Run file triggers in this package other package(s) set off. */
-	    rc = runImmedFileTriggers(psm->ts, psm->te, RPMSENSE_TRIGGERIN,
-				    RPMSCRIPT_FILETRIGGER, 2);
+	    rc = runImmedFileTriggers(psm->ts, psm->te, psm->scriptArg,
+				    RPMSENSE_TRIGGERIN, RPMSCRIPT_FILETRIGGER, 2);
 	    if (rc) break;
 	}
 
@@ -747,12 +944,12 @@ static rpmRC rpmPackageErase(rpmts ts, rpmpsm psm)
 
 	if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_NOTRIGGERUN)) {
 	    /* Run file triggers in this package other package(s) set off. */
-	    rc = runImmedFileTriggers(psm->ts, psm->te, RPMSENSE_TRIGGERUN,
-				    RPMSCRIPT_FILETRIGGER, 1);
+	    rc = runImmedFileTriggers(psm->ts, psm->te, psm->scriptArg,
+				    RPMSENSE_TRIGGERUN, RPMSCRIPT_FILETRIGGER, 1);
 	    if (rc) break;
 
 	    /* Run file triggers in other package(s) this package sets off. */
-	    rc = runFileTriggers(psm->ts, psm->te, RPMSENSE_TRIGGERUN,
+	    rc = runFileTriggers(psm->ts, psm->te, psm->scriptArg, RPMSENSE_TRIGGERUN,
 				RPMSCRIPT_FILETRIGGER, 1);
 	    if (rc) break;
 
@@ -772,17 +969,18 @@ static rpmRC rpmPackageErase(rpmts ts, rpmpsm psm)
 
 	if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_NOTRIGGERUN)) {
 	    /* Run file triggers in this package other package(s) set off. */
-	    rc = runImmedFileTriggers(psm->ts, psm->te, RPMSENSE_TRIGGERUN,
-				    RPMSCRIPT_FILETRIGGER, 2);
+	    rc = runImmedFileTriggers(psm->ts, psm->te, psm->scriptArg,
+				    RPMSENSE_TRIGGERUN, RPMSCRIPT_FILETRIGGER, 2);
 	    if (rc) break;
 
 	    /* Run file triggers in other package(s) this package sets off. */
-	    rc = runFileTriggers(psm->ts, psm->te, RPMSENSE_TRIGGERUN,
+	    rc = runFileTriggers(psm->ts, psm->te, psm->scriptArg, RPMSENSE_TRIGGERUN,
 				RPMSCRIPT_FILETRIGGER, 2);
 	    if (rc) break;
 	}
 
-	if ((rc = rpmChrootIn()) == 0) {
+	rc = rpmChrootIn() ? RPMRC_FAIL : RPMRC_OK;
+	if (rc == RPMRC_OK) {
 	    rc = rpmpsmRemove(psm);
 	    rpmChrootOut();
 	}
@@ -790,7 +988,7 @@ static rpmRC rpmPackageErase(rpmts ts, rpmpsm psm)
 
 	/* Run file triggers in other package(s) this package sets off. */
 	if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_NOTRIGGERPOSTUN)) {
-	    rc = runFileTriggers(psm->ts, psm->te, RPMSENSE_TRIGGERPOSTUN,
+	    rc = runFileTriggers(psm->ts, psm->te, psm->scriptArg, RPMSENSE_TRIGGERPOSTUN,
 				RPMSCRIPT_FILETRIGGER, 1);
 	}
 
@@ -805,7 +1003,7 @@ static rpmRC rpmPackageErase(rpmts ts, rpmpsm psm)
 	    if (rc) break;
 
 	    /* Run file triggers in other package(s) this package sets off. */
-	    rc = runFileTriggers(psm->ts, psm->te, RPMSENSE_TRIGGERPOSTUN,
+	    rc = runFileTriggers(psm->ts, psm->te, psm->scriptArg, RPMSENSE_TRIGGERPOSTUN,
 				RPMSCRIPT_FILETRIGGER, 2);
 	}
 	if (rc) break;
@@ -815,7 +1013,8 @@ static rpmRC rpmPackageErase(rpmts ts, rpmpsm psm)
 	    rpmtriggersPrepPostUnTransFileTrigs(psm->ts, psm->te);
 	}
 
-	rc = dbRemove(ts, psm->te);
+	if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_NODB))
+	    rc = dbRemove(ts, psm->te);
     }
 
     rpmswExit(rpmtsOp(psm->ts, RPMTS_OP_ERASE), 0);
@@ -823,14 +1022,49 @@ static rpmRC rpmPackageErase(rpmts ts, rpmpsm psm)
     return rc;
 }
 
+static rpmRC rpmPackageRestore(rpmts ts, rpmpsm psm)
+{
+    rpmRC rc = RPMRC_OK;
+
+    rpmswEnter(rpmtsOp(psm->ts, RPMTS_OP_INSTALL), 0);
+    rc = rpmChrootIn() ? RPMRC_FAIL : RPMRC_OK;
+    if (rc == RPMRC_OK) {
+	char *failedFile = NULL;
+	rpmpsmNotify(psm, RPMCALLBACK_INST_START, 0);
+	/* make sure first progress call gets made */
+	rpmpsmNotify(psm, RPMCALLBACK_INST_PROGRESS, 0);
+
+	if (!(rpmtsFlags(psm->ts) & RPMTRANS_FLAG_JUSTDB)) {
+	    if (rpmfilesFC(psm->files) > 0) {
+		int fsmrc = rpmPackageFilesInstall(psm->ts, psm->te, psm->files,
+					      psm, &failedFile);
+		rc = (fsmrc == 0) ? RPMRC_OK : RPMRC_FAIL;
+	    }
+	}
+
+	/* XXX make sure progress reaches 100% */
+	rpmpsmNotify(psm, RPMCALLBACK_INST_PROGRESS, psm->total);
+	rpmpsmNotify(psm, RPMCALLBACK_INST_STOP, psm->total);
+
+	free(failedFile);
+	rpmChrootOut();
+    }
+    rpmswExit(rpmtsOp(psm->ts, RPMTS_OP_INSTALL), 0);
+
+    return rc;
+}
+
 static const char * pkgGoalString(pkgGoal goal)
 {
     switch (goal) {
     case PKG_INSTALL:	return "  install";
     case PKG_ERASE:	return "    erase";
+    case PKG_RESTORE:	return "  restore";
     case PKG_VERIFY:	return "   verify";
     case PKG_PRETRANS:	return " pretrans";
     case PKG_POSTTRANS:	return "posttrans";
+    case PKG_PREUNTRANS:	return " preuntrans";
+    case PKG_POSTUNTRANS:	return "postuntrans";
     default:		return "unknown";
     }
 }
@@ -845,18 +1079,23 @@ static rpmRC runGoal(rpmpsm psm, pkgGoal goal)
     case PKG_ERASE:
 	rc = rpmPackageErase(psm->ts, psm);
 	break;
+    case PKG_RESTORE:
+	rc = rpmPackageRestore(psm->ts, psm);
+	break;
     case PKG_PRETRANS:
     case PKG_POSTTRANS:
+    case PKG_PREUNTRANS:
+    case PKG_POSTUNTRANS:
     case PKG_VERIFY:
 	rc = runInstScript(psm, goal);
 	break;
     case PKG_TRANSFILETRIGGERIN:
-	rc = runImmedFileTriggers(psm->ts, psm->te, RPMSENSE_TRIGGERIN,
-				    RPMSCRIPT_TRANSFILETRIGGER, 0);
+	rc = runImmedFileTriggers(psm->ts, psm->te, psm->scriptArg,
+				RPMSENSE_TRIGGERIN, RPMSCRIPT_TRANSFILETRIGGER, 0);
 	break;
     case PKG_TRANSFILETRIGGERUN:
-	rc = runImmedFileTriggers(psm->ts, psm->te, RPMSENSE_TRIGGERUN,
-				    RPMSCRIPT_TRANSFILETRIGGER, 0);
+	rc = runImmedFileTriggers(psm->ts, psm->te, psm->scriptArg,
+				RPMSENSE_TRIGGERUN, RPMSCRIPT_TRANSFILETRIGGER, 0);
 	break;
     default:
 	break;
