@@ -7,13 +7,15 @@
 
 #include <rpm/rpmte.h>
 #include <rpm/rpmfi.h>
+#include <rpm/rpmstring.h>
 #include <rpm/rpmstrpool.h>
 
-#include "lib/rpmal.h"
-#include "lib/misc.h"
-#include "lib/rpmte_internal.h"
-#include "lib/rpmds_internal.h"
-#include "lib/rpmfi_internal.h"
+#include "rpmal.h"
+#include "misc.h"
+#include "rpmte_internal.h"
+#include "rpmds_internal.h"
+#include "rpmfi_internal.h"
+#include "rpmts_internal.h"
 
 #include "debug.h"
 
@@ -38,14 +40,14 @@ typedef struct availableIndexEntry_s {
     unsigned int entryIx;	/*!< Dependency index. */
 } * availableIndexEntry;
 
-#undef HASHTYPE
-#undef HTKEYTYPE
-#undef HTDATATYPE
 #define HASHTYPE rpmalDepHash
 #define HTKEYTYPE rpmsid
 #define HTDATATYPE struct availableIndexEntry_s
-#include "lib/rpmhash.H"
-#include "lib/rpmhash.C"
+#include "rpmhash.H"
+#include "rpmhash.C"
+#undef HASHTYPE
+#undef HTKEYTYPE
+#undef HTDATATYPE
 
 typedef struct availableIndexFileEntry_s {
     rpmsid dirName;
@@ -53,14 +55,14 @@ typedef struct availableIndexFileEntry_s {
     unsigned int entryIx;	/*!< Dependency index. */
 } * availableIndexFileEntry;
 
-#undef HASHTYPE
-#undef HTKEYTYPE
-#undef HTDATATYPE
 #define HASHTYPE rpmalFileHash
 #define HTKEYTYPE rpmsid
 #define HTDATATYPE struct availableIndexFileEntry_s
-#include "lib/rpmhash.H"
-#include "lib/rpmhash.C"
+#include "rpmhash.H"
+#include "rpmhash.C"
+#undef HASHTYPE
+#undef HTKEYTYPE
+#undef HTDATATYPE
 
 /** \ingroup rpmdep
  * Set of available packages, items, and directories.
@@ -92,26 +94,22 @@ static void rpmalFreeIndex(rpmal al)
     al->fpc = fpCacheFree(al->fpc);
 }
 
-rpmal rpmalCreate(rpmstrPool pool, int delta, rpmtransFlags tsflags,
-		  rpm_color_t tscolor, rpm_color_t prefcolor)
+rpmal rpmalCreate(rpmts ts, int delta)
 {
-    rpmal al = xcalloc(1, sizeof(*al));
+    rpmal al = (rpmal)xcalloc(1, sizeof(*al));
 
-    /* transition time safe-guard */
-    assert(pool != NULL);
-
-    al->pool = rpmstrPoolLink(pool);
+    al->pool = rpmstrPoolLink(rpmtsPool(ts));
     al->delta = delta;
     al->size = 0;
     al->alloced = al->delta;
-    al->list = xmalloc(sizeof(*al->list) * al->alloced);;
+    al->list = (availablePackage)xmalloc(sizeof(*al->list) * al->alloced);
 
     al->providesHash = NULL;
     al->obsoletesHash = NULL;
     al->fileHash = NULL;
-    al->tsflags = tsflags;
-    al->tscolor = tscolor;
-    al->prefcolor = prefcolor;
+    al->tsflags = rpmtsFlags(ts);
+    al->tscolor = rpmtsColor(ts);
+    al->prefcolor = rpmtsPrefColor(ts);
 
     return al;
 }
@@ -250,6 +248,10 @@ void rpmalAdd(rpmal al, rpmte p)
     rpmalNum pkgNum;
     availablePackage alp;
 
+    /* Source packages don't provide anything to depsolving */
+    if (rpmteIsSource(p))
+	return;
+
     if (al->size == al->alloced) {
 	al->alloced += al->delta;
 	al->list = xrealloc(al->list, sizeof(*al->list) * al->alloced);
@@ -264,22 +266,6 @@ void rpmalAdd(rpmal al, rpmte p)
     alp->obsoletes = rpmdsLink(rpmteDS(p, RPMTAG_OBSOLETENAME));
     alp->fi = rpmteFiles(p);
 
-    /*
-     * Transition-time safe-guard to catch private-pool uses.
-     * File sets with no files have NULL pool, that's fine. But WTF is up
-     * with the provides: every single package should have at least its
-     * own name as a provide, and thus never NULL, and normal use matches
-     * this expectation. However the test-suite is tripping up on NULL
-     * NULL pool from NULL alp->provides in numerous cases?
-     */
-    {
-	rpmstrPool fipool = rpmfilesPool(alp->fi);
-	rpmstrPool dspool = rpmdsPool(alp->provides);
-	
-	assert(fipool == NULL || fipool == al->pool);
-	assert(dspool == NULL || dspool == al->pool);
-    }
-
     /* Try to be lazy as delayed hash creation is cheaper */
     if (al->providesHash != NULL)
 	rpmalAddProvides(al, pkgNum, alp->provides);
@@ -287,8 +273,6 @@ void rpmalAdd(rpmal al, rpmte p)
 	rpmalAddObsoletes(al, pkgNum, alp->obsoletes);
     if (al->fileHash != NULL)
 	rpmalAddFiles(al, pkgNum, alp->fi);
-
-    assert(((rpmalNum)(alp - al->list)) == pkgNum);
 }
 
 static void rpmalMakeFileIndex(rpmal al)
@@ -364,7 +348,7 @@ rpmte * rpmalAllObsoletes(rpmal al, rpmds ds)
 	availablePackage alp;
 	int rc, found = 0;
 
-	ret = xmalloc((resultCnt+1) * sizeof(*ret));
+	ret = (rpmte *)xmalloc((resultCnt+1) * sizeof(*ret));
 
 	for (int i = 0; i < resultCnt; i++) {
 	    alp = al->list + result[i].pkgNum;
@@ -416,14 +400,10 @@ static rpmte * rpmalAllFileSatisfiesDepend(const rpmal al, const char *fileName,
 
 	if (resultCnt > 0) {
 	    int i, found;
-	    ret = xmalloc((resultCnt+1) * sizeof(*ret));
+	    ret = (rpmte *)xmalloc((resultCnt+1) * sizeof(*ret));
 	    fingerPrint * fp = NULL;
 	    rpmsid dirName = rpmstrPoolIdn(al->pool, fileName, bnStart, 1);
 
-	    if (!al->fpc)
-		al->fpc = fpCacheCreate(1001, NULL);
-	    fpLookup(al->fpc, rpmstrPoolStr(al->pool, dirName), fileName + bnStart, &fp);
-
 	    for (found = i = 0; i < resultCnt; i++) {
 		availablePackage alp = al->list + result[i].pkgNum;
 		if (alp->p == NULL) /* deleted */
@@ -431,10 +411,15 @@ static rpmte * rpmalAllFileSatisfiesDepend(const rpmal al, const char *fileName,
 		/* ignore self-conflicts/obsoletes */
 		if (filterds && rpmteDS(alp->p, rpmdsTagN(filterds)) == filterds)
 		    continue;
-		if (result[i].dirName != dirName &&
-		    !fpLookupEquals(al->fpc, fp, rpmstrPoolStr(al->pool, result[i].dirName), fileName + bnStart))
-		    continue;
-
+		if (result[i].dirName != dirName) {
+		    /* if the directory is different check the fingerprints */
+		    if (!al->fpc)
+			al->fpc = fpCacheCreate(1001, al->pool);
+		    if (!fp)
+			fpLookupId(al->fpc, dirName, baseName, &fp);
+		    if (!fpLookupEqualsId(al->fpc, fp, result[i].dirName, baseName))
+			continue;
+		}
 		ret[found] = alp->p;
 		found++;
 	    }
@@ -488,7 +473,7 @@ rpmte * rpmalAllSatisfiesDepend(const rpmal al, const rpmds ds)
 
     if (resultCnt==0) return NULL;
 
-    ret = xmalloc((resultCnt+1) * sizeof(*ret));
+    ret = (rpmte *)xmalloc((resultCnt+1) * sizeof(*ret));
 
     for (found=i=0; i<resultCnt; i++) {
 	alp = al->list + result[i].pkgNum;
