@@ -14,13 +14,14 @@
 #include <rpm/rpmbase64.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <inttypes.h>
 
-#include "lib/rpmfi_internal.h"
-#include "lib/rpmte_internal.h"	/* relocations */
-#include "lib/cpio.h"	/* XXX CPIO_FOO */
-#include "lib/fsm.h"	/* rpmpsm stuff for now */
-#include "lib/rpmug.h"
-#include "rpmio/rpmio_internal.h"       /* fdInit/FiniDigest */
+#include "rpmfi_internal.h"
+#include "rpmte_internal.h"	/* relocations */
+#include "cpio.h"	/* XXX CPIO_FOO */
+#include "fsm.h"	/* rpmpsm stuff for now */
+#include "rpmug.h"
+#include "rpmio_internal.h"       /* fdInit/FiniDigest */
 
 #include "debug.h"
 
@@ -31,14 +32,11 @@ struct hardlinks_s {
 
 typedef struct hardlinks_s * hardlinks_t;
 
-#undef HASHTYPE
-#undef HTKEYTYPE
-#undef HTDATATYPE
 #define HASHTYPE nlinkHash
 #define HTKEYTYPE int
 #define HTDATATYPE struct hardlinks_s *
-#include "lib/rpmhash.H"
-#include "lib/rpmhash.C"
+#include "rpmhash.H"
+#include "rpmhash.C"
 #undef HASHTYPE
 #undef HTKEYTYPE
 #undef HTDATATYPE
@@ -60,7 +58,7 @@ struct rpmfi_s {
 
     rpmfiles files;		/*!< File info set */
     rpmcpio_t archive;		/*!< Archive with payload */
-    unsigned char * found;	/*!< Bit field of files found in the archive */
+    uint8_t * found;	/*!< Bit field of files found in the archive */
     int nrefs;			/*!< Reference count */
 };
 
@@ -127,8 +125,7 @@ struct rpmfiles_s {
     unsigned char * veritysigs; /*!< Verity signatures in binary. */
 
     struct nlinkHash_s * nlinks;/*!< Files connected by hardlinks */
-    rpm_off_t * replacedSizes;	/*!< (TR_ADDED) */
-    rpm_loff_t * replacedLSizes;/*!< (TR_ADDED) */
+    rpm_loff_t * replacedSizes;	/*!< (TR_ADDED) */
     int magic;
     int nrefs;		/*!< Reference count. */
 };
@@ -196,7 +193,7 @@ static int rpmfnInit(rpmfn fndata, rpmTagVal bntag, Header h, rpmstrPool pool)
 	    fndata->bnid = rpmtdToPool(&bn, pool);
 	    fndata->dnid = rpmtdToPool(&dn, pool);
 	    /* Steal index data from the td (pooh...) */
-	    fndata->dil = dx.data;
+	    fndata->dil = (uint32_t *)dx.data;
 	    dx.data = NULL;
 	    rc = fndata->fc;
 	} else {
@@ -328,7 +325,6 @@ int rpmfiSetFX(rpmfi fi, int fx)
 
     if (fi != NULL && fx >= 0 && fx < rpmfilesFC(fi->files)) {
 	int dx = fi->j;
-	i = fi->i;
 	fi->i = fx;
 	fi->j = rpmfilesDI(fi->files, fi->i);
 	i = fi->i;
@@ -347,11 +343,6 @@ int rpmfiDX(rpmfi fi)
     return (fi != NULL ? fi->j : -1);
 }
 
-int rpmfiSetDX(rpmfi fi, int dx)
-{
-    return -1;
-}
-
 int rpmfilesDI(rpmfiles fi, int ix)
 {
     return (fi != NULL) ? rpmfnDI(&fi->fndata, ix) : -1;
@@ -546,7 +537,7 @@ rpmfileState rpmfilesFState(rpmfiles fi, int ix)
 
     if (fi != NULL && ix >= 0 && ix < rpmfilesFC(fi)) {
 	if (fi->fstates != NULL)
-	    fstate = fi->fstates[ix];
+	    fstate = (rpmfileState)fi->fstates[ix];
     }
     return fstate;
 }
@@ -899,16 +890,6 @@ rpmfi rpmfiInit(rpmfi fi, int fx)
     return fi;
 }
 
-int rpmfiNextD(rpmfi fi)
-{
-    return -1;
-}
-
-rpmfi rpmfiInitD(rpmfi fi, int dx)
-{
-    return NULL;
-}
-
 rpmFileTypes rpmfiWhatis(rpm_mode_t mode)
 {
     if (S_ISDIR(mode))	return XDIR;
@@ -1048,8 +1029,8 @@ rpmFileAction rpmfilesDecideFate(rpmfiles ofi, int oix,
     char buffer[1024];
     rpmFileTypes dbWhat, newWhat, diskWhat;
     struct stat sb;
-    int save = (newFlags & RPMFILE_NOREPLACE) ? FA_ALTNAME : FA_SAVE;
-    int action = FA_CREATE; /* assume we can create */
+    rpmFileAction save = (newFlags & RPMFILE_NOREPLACE) ? FA_ALTNAME : FA_SAVE;
+    rpmFileAction action = FA_CREATE; /* assume we can create */
 
     /* If the new file is a ghost, leave whatever might be on disk alone. */
     if (newFlags & RPMFILE_GHOST) {
@@ -1187,11 +1168,10 @@ int rpmfilesConfigConflict(rpmfiles fi, int ix)
     if (!(flags & RPMFILE_CONFIG))
 	return 0;
 
-    /* Only links and regular files can be %config, this is kinda moot */
-    /* XXX: Why are we returning 1 here? */
+    /* Only links and regular files can be %config */
     newWhat = rpmfiWhatis(rpmfilesFMode(fi, ix));
     if (newWhat != LINK && newWhat != REG)
-	return 1;
+	return 0;
 
     /* If it's not on disk, there's nothing to be saved */
     fn = rpmfilesFN(fi, ix);
@@ -1302,7 +1282,6 @@ rpmfiles rpmfilesFree(rpmfiles fi)
     }
 
     fi->replacedSizes = _free(fi->replacedSizes);
-    fi->replacedLSizes = _free(fi->replacedLSizes);
 
     fi->h = headerFree(fi->h);
     fi->pool = rpmstrPoolFree(fi->pool);
@@ -1380,15 +1359,15 @@ static int indexSane(rpmtd xd, rpmtd yd, rpmtd zd)
 	} \
 	if (rpmTagGetTagType(_tag) != RPM_STRING_ARRAY_TYPE && rpmTagGetTagType(_tag) != RPM_I18NSTRING_TYPE && \
 	    (_td)->size < totalfc * sizeof(*(_data))) {		\
-	    rpmlog(RPMLOG_ERR, _("Malformed data for tag %s: %u bytes found but %lu expected.\n"), rpmTagGetName(_tag), (_td)->size, totalfc * sizeof(*(_data))); \
+	    rpmlog(RPMLOG_ERR, _("Malformed data for tag %s: %u bytes found but %" PRIu64 " expected.\n"), rpmTagGetName(_tag), (_td)->size, (uint64_t)totalfc * sizeof(*(_data))); \
 	    goto err;				\
 	} \
-	_data = ((_td)->data); \
+	_data = (typeof((_data)))((_td)->data); \
     }
 /* Get file data from header without checking number of entries */
 #define _hgfinc(_h, _tag, _td, _flags, _data) \
     if (headerGet((_h), (_tag), (_td), (_flags))) {\
-	_data = ((_td)->data);	   \
+	_data = (typeof((_data)))((_td)->data);	   \
     }
 
 /*** Hard link handling ***/
@@ -1398,14 +1377,11 @@ struct fileid_s {
     rpm_ino_t id_ino;
 };
 
-#undef HASHTYPE
-#undef HTKEYTYPE
-#undef HTDATATYPE
 #define HASHTYPE fileidHash
 #define HTKEYTYPE struct fileid_s
 #define HTDATATYPE int
-#include "lib/rpmhash.H"
-#include "lib/rpmhash.C"
+#include "rpmhash.H"
+#include "rpmhash.C"
 #undef HASHTYPE
 #undef HTKEYTYPE
 #undef HTDATATYPE
@@ -1483,7 +1459,7 @@ static void rpmfilesBuildNLink(rpmfiles fi, Header h)
 	    fileidHashGetEntry(files, f_id, &data, &fcnt, NULL);
 	    if (fcnt > 1 && !nlinkHashHasEntry(fi->nlinks, i)) {
 		struct hardlinks_s * hlinks;
-		hlinks = xmalloc(sizeof(struct hardlinks_s)+
+		hlinks = (struct hardlinks_s *)xmalloc(sizeof(*hlinks) +
 			fcnt*sizeof(hlinks->files[0]));
 		hlinks->nlink = fcnt;
 		for (int j=0; j<fcnt; j++) {
@@ -1516,15 +1492,15 @@ static uint8_t *hex2binv(Header h, rpmTagVal tag, rpm_count_t num,
     if (headerGet(h, tag, &td, HEADERGET_MINMEM) && rpmtdCount(&td) == num) {
 	const char *s;
 	int i = 0;
-	uint8_t *t = bin = xmalloc(((rpmtdSize(&td) / 2) + 1));
-	offs = xmalloc((num + 1) * sizeof(*offs));
+	uint8_t *t = bin = (uint8_t *)xmalloc(((rpmtdSize(&td) / 2) + 1));
+	offs = (uint32_t *)xmalloc((num + 1) * sizeof(*offs));
 
 	while ((s = rpmtdNextString(&td))) {
 	    uint32_t slen = strlen(s);
 	    uint32_t len = slen / 2;
 	    if (slen % 2) {
-		bin = rfree(bin);
-		offs = rfree(offs);
+		bin = _free(bin);
+		offs = _free(offs);
 		goto exit;
 	    }
 	    offs[i] = t - bin;
@@ -1548,7 +1524,7 @@ static uint8_t *hex2bin(Header h, rpmTagVal tag, rpm_count_t num, size_t len)
     uint8_t *bin = NULL;
 
     if (headerGet(h, tag, &td, HEADERGET_MINMEM) && rpmtdCount(&td) == num) {
-	uint8_t *t = bin = xmalloc(num * len);
+	uint8_t *t = bin = (uint8_t *)xmalloc(num * len);
 	const char *s;
 
 	while ((s = rpmtdNextString(&td))) {
@@ -1558,7 +1534,7 @@ static uint8_t *hex2bin(Header h, rpmTagVal tag, rpm_count_t num, size_t len)
 		continue;
 	    }
 	    if (strlen(s) != len * 2) {
-		bin = rfree(bin);
+		bin = _free(bin);
 		break;
 	    }
 	    for (int j = 0; j < len; j++, t++, s += 2)
@@ -1582,8 +1558,8 @@ static uint8_t *base2bin(Header h, rpmTagVal tag, rpm_count_t num, int *len)
     uint8_t *bin = NULL, *t = NULL;
     size_t maxlen = 0;
     int status, i= 0;
-    void **arr = xmalloc(num * sizeof(void *));
-    size_t *lengths = xcalloc(num, sizeof(size_t));
+    uint8_t **arr = (uint8_t **)xmalloc(num * sizeof(void *));
+    size_t *lengths = (size_t *)xcalloc(num, sizeof(size_t));
     const char *s;
 
     if (!headerGet(h, tag, &td, HEADERGET_MINMEM) || rpmtdCount(&td) != num)
@@ -1595,11 +1571,11 @@ static uint8_t *base2bin(Header h, rpmTagVal tag, rpm_count_t num, int *len)
 	    arr[i++] = NULL;
 	    continue;
 	}
-	status = rpmBase64Decode(s, &arr[i], &lengths[i]);
+	status = rpmBase64Decode(s, (void**)&arr[i], &lengths[i]);
 	if (lengths[i] > maxlen)
 	    maxlen = lengths[i];
 	if (status) {
-	    rpmlog(RPMLOG_DEBUG, _("%s: base64 decode failed, len %li\n"),
+	    rpmlog(RPMLOG_DEBUG, _("%s: base64 decode failed, len %zu\n"),
 		   __func__, lengths[i]);
 	    goto out;
 	}
@@ -1607,10 +1583,10 @@ static uint8_t *base2bin(Header h, rpmTagVal tag, rpm_count_t num, int *len)
     }
 
     if (maxlen) {
-	rpmlog(RPMLOG_DEBUG, _("%s: base64 decode success, len %li\n"),
+	rpmlog(RPMLOG_DEBUG, _("%s: base64 decode success, len %zu\n"),
 	       __func__, maxlen);
 
-	t = bin = xcalloc(num, maxlen);
+	t = bin = (uint8_t *)xcalloc(num, maxlen);
 
 	for (i = 0; i < num; i++) {
 	    if (arr[i]) {
@@ -1731,7 +1707,7 @@ static int rpmfilesPopulate(rpmfiles fi, Header h, rpmfiFlags flags)
 
 rpmfiles rpmfilesNew(rpmstrPool pool, Header h, rpmTagVal tagN, rpmfiFlags flags)
 {
-    rpmfiles fi = xcalloc(1, sizeof(*fi)); 
+    rpmfiles fi = (rpmfiles)xcalloc(1, sizeof(*fi));
     int fc;
 
     fi->magic = RPMFIMAGIC;
@@ -1754,7 +1730,7 @@ rpmfiles rpmfilesNew(rpmstrPool pool, Header h, rpmTagVal tagN, rpmfiFlags flags
 	if (headerIsEntry(h, RPMTAG_ORIGBASENAMES)) {
 	    /* For relocated packages, grab the original paths too */
 	    int ofc;
-	    fi->ofndata = xmalloc(sizeof(*fi->ofndata));
+	    fi->ofndata = (rpmfn)xmalloc(sizeof(*fi->ofndata));
 	    ofc = rpmfnInit(fi->ofndata, RPMTAG_ORIGBASENAMES, h, fi->pool);
 	    
 	    if (ofc != 0 && ofc != fc)
@@ -1802,7 +1778,7 @@ static rpmfi initIter(rpmfiles files, int itype, int link)
     rpmfi fi = NULL;
 
     if (files && itype>=0 && itype<=RPMFILEITERMAX) {
-	fi = xcalloc(1, sizeof(*fi)); 
+	fi = (rpmfi)xcalloc(1, sizeof(*fi));
 	fi->i = -1;
 	fi->j = -1;
 	fi->files = link ? rpmfilesLink(files) : files;
@@ -1812,7 +1788,7 @@ static rpmfi initIter(rpmfiles files, int itype, int link)
 	} else if (itype >=RPMFI_ITER_READ_ARCHIVE
 	    && itype <= RPMFI_ITER_READ_ARCHIVE_OMIT_HARDLINKS) {
 
-	    fi->found = xcalloc(1, (rpmfiFC(fi)>>3) + 1);
+	    fi->found = (uint8_t *)xcalloc(1, (rpmfiFC(fi)>>3) + 1);
 	}
 	rpmfiLink(fi);
     }
@@ -1887,22 +1863,9 @@ void rpmfilesSetFReplacedSize(rpmfiles fi, int ix, rpm_loff_t newsize)
     if (fi != NULL && ix >= 0 && ix < rpmfilesFC(fi)) {
 	/* Switch over to 64 bit variant */
 	int fc = rpmfilesFC(fi);
-	if (newsize > UINT32_MAX && fi->replacedLSizes == NULL) {
-	    fi->replacedLSizes = xcalloc(fc, sizeof(*fi->replacedLSizes));
-	    /* copy 32 bit data */
-	    if (fi->replacedSizes) {
-		for (int i=0; i < fc; i++)
-		    fi->replacedLSizes[i] = fi->replacedSizes[i];
-		fi->replacedSizes = _free(fi->replacedSizes);
-	    }
-	}
-	if (fi->replacedLSizes != NULL) {
-	    fi->replacedLSizes[ix] = newsize;
-	} else {
-	    if (fi->replacedSizes == NULL)
-		fi->replacedSizes = xcalloc(fc, sizeof(*fi->replacedSizes));
-	    fi->replacedSizes[ix] = (rpm_off_t) newsize;
-	}
+	if (fi->replacedSizes == NULL)
+	    fi->replacedSizes = (rpm_loff_t *)xcalloc(fc, sizeof(*fi->replacedSizes));
+	fi->replacedSizes[ix] = newsize;
     }
 }
 
@@ -1912,8 +1875,6 @@ rpm_loff_t rpmfilesFReplacedSize(rpmfiles fi, int ix)
     if (fi != NULL && ix >= 0 && ix < rpmfilesFC(fi)) {
 	if (fi->replacedSizes) {
 	    rsize = fi->replacedSizes[ix];
-	} else if (fi->replacedLSizes) {
-	    rsize = fi->replacedLSizes[ix];
 	}
     }
     return rsize;
@@ -2300,7 +2261,7 @@ static int iterReadArchiveNext(rpmfi fi)
 	} else if (S_ISLNK(mode)) {
 	    /* Skip over symlink target data in payload */
 	    rpm_loff_t lsize = rpmfilesFSize(fi->files, fx);
-	    char *buf = xmalloc(lsize + 1);
+	    char *buf = (char *)xmalloc(lsize + 1);
 	    if (rpmcpioRead(fi->archive, buf, lsize) != lsize)
 		rc = RPMERR_READ_FAILED;
 	    /* XXX should we validate the payload matches? */
@@ -2406,7 +2367,7 @@ int rpmfiArchiveReadToFilePsm(rpmfi fi, FD_t fd, int nodigest, rpmpsm psm)
 
     rpm_loff_t left = rpmfiFSize(fi);
     const unsigned char * fidigest = NULL;
-    rpmHashAlgo digestalgo = 0;
+    int digestalgo = 0;
     int rc = 0;
     char buf[BUFSIZ*4];
 
@@ -2507,7 +2468,7 @@ char * rpmfileStrerror(int rc)
     case RPMERR_DIGEST_MISMATCH: s = _("Digest mismatch");	break;
     case RPMERR_INTERNAL:	s = _("Internal error");	break;
     case RPMERR_UNMAPPED_FILE:	s = _("Archive file not in header"); break;
-    case RPMERR_INVALID_SYMLINK: s = _("Invalid symlink");	break;
+    case RPMERR_INVALID_SYMLINK: s = _("Unsafe symlink");	break;
     case RPMERR_ENOTDIR:	s = strerror(ENOTDIR);	break;
     case RPMERR_ENOENT:	s = strerror(ENOENT); break;
     case RPMERR_ENOTEMPTY:	s = strerror(ENOTEMPTY); break;
