@@ -16,8 +16,8 @@
 #include <rpm/rpmlog.h>
 #include <rpm/rpmfileutil.h>
 
-#include "lib/rpmgi.h"
-#include "lib/manifest.h"
+#include "rpmgi.h"
+#include "manifest.h"
 #include "debug.h"
 
 static int rpmcliPackagesTotal = 0;
@@ -417,7 +417,7 @@ static int rpmNoGlob(const char *fn, int *argcPtr, ARGV_t * argvPtr)
 /** @todo Generalize --freshen policies. */
 int rpmInstall(rpmts ts, struct rpmInstallArguments_s * ia, ARGV_t fileArgv)
 {
-    struct rpmEIU * eiu = xcalloc(1, sizeof(*eiu));
+    struct rpmEIU * eiu = (struct rpmEIU *)xcalloc(1, sizeof(*eiu));
     rpmRelocation * relocations;
     char * fileURL = NULL;
     rpmVSFlags vsflags, ovsflags;
@@ -459,9 +459,7 @@ int rpmInstall(rpmts ts, struct rpmInstallArguments_s * ia, ARGV_t fileArgv)
 	if (giFlags & RPMGI_NOGLOB) {
 	    rc = rpmNoGlob(*eiu->fnp, &ac, &av);
 	} else {
-	    char * fn = rpmEscapeSpaces(*eiu->fnp);
-	    rc = rpmGlob(fn, &ac, &av);
-	    fn = _free(fn);
+	    rc = rpmGlobPath(*eiu->fnp, RPMGLOB_NOCHECK, &ac, &av);
 	}
 	if (rc || ac == 0) {
 	    if (giFlags & RPMGI_NOGLOB) {
@@ -663,6 +661,7 @@ restart:
 	rpmcliProgressState = 0;
 	rpmcliProgressTotal = 0;
 	rpmcliProgressCurrent = 0;
+	rpmtsEmpty(ts);
 	for (i = 0; i < eiu->numSRPMS; i++) {
 	    if (eiu->sourceURL[i] != NULL) {
 	        rc = RPMRC_OK;
