@@ -7,7 +7,7 @@
 #include <rpm/rpmstring.h>
 #include <rpm/rpmts.h>
 
-#include "lib/rpmplugins.h"
+#include "rpmplugins.h"
 #include <dlfcn.h>
 
 
@@ -49,11 +49,19 @@ int rpmpluginsPluginAdded(rpmPlugins plugins, const char *name)
 
 rpmPlugins rpmpluginsNew(rpmts ts)
 {
-    rpmPlugins plugins = xcalloc(1, sizeof(*plugins));
+    rpmPlugins plugins = (rpmPlugins)xcalloc(1, sizeof(*plugins));
     plugins->ts = ts;
     return plugins;
 }
 
+/* ASAN upstream recommends avoiding dlclose() */
+static void mydlclose(void *handle)
+{
+#ifndef ENABLE_ASAN
+    dlclose(handle);
+#endif
+}
+
 static rpmPlugin rpmPluginNew(const char *name, const char *path,
 			      const char *opts)
 {
@@ -72,13 +80,13 @@ static rpmPlugin rpmPluginNew(const char *name, const char *path,
     hooks_name = rstrscat(NULL, name, "_hooks", NULL);
     /* clear out any old errors that weren't fetched */
     dlerror();
-    hooks = dlsym(handle, hooks_name);
+    hooks = (rpmPluginHooks)dlsym(handle, hooks_name);
     if ((error = dlerror()) != NULL) {
 	rpmlog(RPMLOG_ERR, _("Failed to resolve symbol %s: %s\n"),
 	       hooks_name, error);
-	dlclose(handle);
+	mydlclose(handle);
     } else {
-	plugin = xcalloc(1, sizeof(*plugin));
+	plugin = (rpmPlugin)xcalloc(1, sizeof(*plugin));
 	plugin->name = xstrdup(name);
 	plugin->handle = handle;
 	plugin->hooks = hooks;
@@ -96,7 +104,7 @@ static rpmPlugin rpmPluginFree(rpmPlugin plugin)
 	rpmPluginHooks hooks = plugin->hooks;
 	if (hooks->cleanup)
 	    hooks->cleanup(plugin);
-	dlclose(plugin->handle);
+	mydlclose(plugin->handle);
 	free(plugin->name);
 	free(plugin->opts);
 	free(plugin);
@@ -350,21 +358,31 @@ rpmRC rpmpluginsCallScriptletPost(rpmPlugins plugins, const char *s_name, int ty
     return rc;
 }
 
+static char *abspath(rpmfi fi, const char *path)
+{
+    if (*path == '/')
+	return xstrdup(path);
+    else
+	return rstrscat(NULL, rpmfiDN(fi), path, NULL);
+}
+
 rpmRC rpmpluginsCallFsmFilePre(rpmPlugins plugins, rpmfi fi, const char *path,
 			       mode_t file_mode, rpmFsmOp op)
 {
     plugin_fsm_file_pre_func hookFunc;
     int i;
     rpmRC rc = RPMRC_OK;
+    char *apath = abspath(fi, path);
 
     for (i = 0; i < plugins->count; i++) {
 	rpmPlugin plugin = plugins->plugins[i];
 	RPMPLUGINS_SET_HOOK_FUNC(fsm_file_pre);
-	if (hookFunc && hookFunc(plugin, fi, path, file_mode, op) == RPMRC_FAIL) {
+	if (hookFunc && hookFunc(plugin, fi, apath, file_mode, op) == RPMRC_FAIL) {
 	    rpmlog(RPMLOG_ERR, "Plugin %s: hook fsm_file_pre failed\n", plugin->name);
 	    rc = RPMRC_FAIL;
 	}
     }
+    free(apath);
 
     return rc;
 }
@@ -375,34 +393,38 @@ rpmRC rpmpluginsCallFsmFilePost(rpmPlugins plugins, rpmfi fi, const char *path,
     plugin_fsm_file_post_func hookFunc;
     int i;
     rpmRC rc = RPMRC_OK;
+    char *apath = abspath(fi, path);
 
     for (i = 0; i < plugins->count; i++) {
 	rpmPlugin plugin = plugins->plugins[i];
 	RPMPLUGINS_SET_HOOK_FUNC(fsm_file_post);
-	if (hookFunc && hookFunc(plugin, fi, path, file_mode, op, res) == RPMRC_FAIL) {
+	if (hookFunc && hookFunc(plugin, fi, apath, file_mode, op, res) == RPMRC_FAIL) {
 	    rpmlog(RPMLOG_WARNING, "Plugin %s: hook fsm_file_post failed\n", plugin->name);
 	}
     }
+    free(apath);
 
     return rc;
 }
 
 rpmRC rpmpluginsCallFsmFilePrepare(rpmPlugins plugins, rpmfi fi,
-				   const char *path, const char *dest,
+				   int fd, const char *path, const char *dest,
 				   mode_t file_mode, rpmFsmOp op)
 {
     plugin_fsm_file_prepare_func hookFunc;
     int i;
     rpmRC rc = RPMRC_OK;
+    char *apath = abspath(fi, path);
 
     for (i = 0; i < plugins->count; i++) {
 	rpmPlugin plugin = plugins->plugins[i];
 	RPMPLUGINS_SET_HOOK_FUNC(fsm_file_prepare);
-	if (hookFunc && hookFunc(plugin, fi, path, dest, file_mode, op) == RPMRC_FAIL) {
+	if (hookFunc && hookFunc(plugin, fi, fd, apath, dest, file_mode, op) == RPMRC_FAIL) {
 	    rpmlog(RPMLOG_ERR, "Plugin %s: hook fsm_file_prepare failed\n", plugin->name);
 	    rc = RPMRC_FAIL;
 	}
     }
+    free(apath);
 
     return rc;
 }
