@@ -24,14 +24,14 @@
 #include <rpm/rpmsq.h>
 #include <rpm/rpmte.h>
 
-#include "rpmio/rpmpgpval.h"
-#include "lib/rpmal.h"
-#include "lib/rpmchroot.h"
-#include "lib/rpmplugins.h"
-#include "lib/rpmts_internal.h"
-#include "lib/rpmte_internal.h"
-#include "lib/misc.h"
-#include "lib/rpmtriggers.h"
+#include "rpmpgpval.h"
+#include "rpmal.h"
+#include "rpmchroot.h"
+#include "rpmplugins.h"
+#include "rpmts_internal.h"
+#include "rpmte_internal.h"
+#include "misc.h"
+#include "rpmtriggers.h"
 
 #include "debug.h"
 
@@ -190,14 +190,15 @@ rpmdbMatchIterator rpmtsInitIterator(const rpmts ts, rpmDbiTagVal rpmtag,
 	loadKeyring(ts);
 
     /* Parse out "N(EVR)" tokens from a label key if present */
-    if (rpmtag == RPMDBI_LABEL && keyp != NULL && strchr(keyp, '(')) {
-	const char *se, *s = keyp;
+    const char *s = (const char *)keyp;
+    if (rpmtag == RPMDBI_LABEL && keyp != NULL && strchr(s, '(')) {
+	const char *se;
 	char *t;
 	size_t slen = strlen(s);
 	int level = 0;
 	int c;
 
-	tmp = xmalloc(slen+1);
+	tmp = (char *)xmalloc(slen+1);
 	keyp = t = tmp;
 	while ((c = *s++) != '\0') {
 	    switch (c) {
@@ -334,8 +335,15 @@ static int loadKeyringFromDB(rpmts ts)
 	struct rpmtd_s pubkeys;
 	const char *key;
 
-	if (!headerGet(h, RPMTAG_PUBKEYS, &pubkeys, HEADERGET_MINMEM))
-	   continue;
+	/* don't allow normal packages named gpg-pubkey */
+	if (headerIsEntry(h, RPMTAG_ARCH) || headerIsEntry(h, RPMTAG_OS) ||
+	    !headerGet(h, RPMTAG_PUBKEYS, &pubkeys, HEADERGET_MINMEM))
+	{
+	    char *nevr = headerGetAsString(h, RPMTAG_NEVR);
+	    rpmlog(RPMLOG_WARNING, _("%s is not a valid public key\n"), nevr);
+	    free(nevr);
+	    continue;
+	}
 
 	while ((key = rpmtdNextString(&pubkeys))) {
 	    uint8_t *pkt;
@@ -507,6 +515,8 @@ static int makePubkeyHeader(rpmts ts, rpmPubkey key, rpmPubkey *subkeys,
     int rc = -1;
     int i;
 
+    memset(&kd, 0, sizeof(kd));
+
     if ((enc = rpmPubkeyBase64(key)) == NULL)
 	goto exit;
 
@@ -616,20 +626,18 @@ rpmRC rpmtsImportPubkey(const rpmts ts, const unsigned char * pkt, size_t pktlen
     if (txn == NULL)
 	return rc;
 
-    krc = pgpPubKeyLint(pkt, pktlen, &lints);
-    if (lints) {
-        if (krc != RPMRC_OK) {
+    if (pgpPubKeyLint(pkt, pktlen, &lints) != RPMRC_OK) {
+	if (lints) {
             rpmlog(RPMLOG_ERR, "%s\n", lints);
-        } else {
-	    /* XXX Hack to ease testing between different backends */
-	    if (rpmIsNormal())
-		rpmlog(RPMLOG_WARNING, "%s\n", lints);
-        }
-        free(lints);
+	    free(lints);
+	}
+	goto exit;
     }
-    if (krc != RPMRC_OK) {
-        rc = krc;
-        goto exit;
+    if (lints) {
+	/* XXX Hack to ease testing between different backends */
+	if (rpmIsNormal())
+	    rpmlog(RPMLOG_WARNING, "%s\n", lints);
+        free(lints);
     }
 
     /* XXX keyring wont load if sigcheck disabled, force it temporarily */
@@ -646,6 +654,8 @@ rpmRC rpmtsImportPubkey(const rpmts ts, const unsigned char * pkt, size_t pktlen
     krc = rpmKeyringAddKey(keyring, pubkey);
     if (krc < 0)
 	goto exit;
+    for (i = 0; i < subkeysCount; i++)
+	rpmKeyringAddKey(keyring, subkeys[i]);
 
     /* If we dont already have the key, make a persistent record of it */
     if (krc == 0) {
@@ -658,14 +668,16 @@ rpmRC rpmtsImportPubkey(const rpmts ts, const unsigned char * pkt, size_t pktlen
 	headerPutUint32(h, RPMTAG_INSTALLTID, &tid, 1);
 
 	/* Add header to database. */
+	rc = RPMRC_OK;
 	if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_TEST)) {
 	    if (ts->keyringtype == KEYRING_FS)
 		rc = rpmtsImportFSKey(txn, h, 0);
 	    else
 		rc = rpmtsImportHeader(txn, h, 0);
 	}
+    } else {
+	rc = RPMRC_OK;		/* already have key */
     }
-    rc = RPMRC_OK;
 
 exit:
     /* Clean up. */
@@ -1190,11 +1202,10 @@ static int vfylevel_init(void)
 
 rpmts rpmtsCreate(void)
 {
-    rpmts ts;
+    rpmts ts = (rpmts)xcalloc(1, sizeof(*ts));
     tsMembers tsmem;
     char *source_date_epoch = NULL;
 
-    ts = xcalloc(1, sizeof(*ts));
     memset(&ts->ops, 0, sizeof(ts->ops));
     (void) rpmswEnter(rpmtsOp(ts, RPMTS_OP_TOTAL), -1);
     ts->dsi = NULL;
@@ -1242,7 +1253,7 @@ rpmts rpmtsCreate(void)
 	free(tmp);
     }
 
-    tsmem = xcalloc(1, sizeof(*ts->members));
+    tsmem = (tsMembers)xcalloc(1, sizeof(*ts->members));
     tsmem->pool = NULL;
     tsmem->delta = 5;
     tsmem->addedPackages = NULL;
@@ -1298,7 +1309,7 @@ rpmtsi rpmtsiInit(rpmts ts)
 {
     rpmtsi tsi = NULL;
 
-    tsi = xcalloc(1, sizeof(*tsi));
+    tsi = (rpmtsi)xcalloc(1, sizeof(*tsi));
     tsi->ts = rpmtsLink(ts);
     tsi->oc = 0;
     return tsi;
@@ -1364,8 +1375,9 @@ rpmtxn rpmtxnBegin(rpmts ts, rpmtxnFlags flags)
     if (ts->lock == NULL)
 	ts->lock = rpmlockNew(ts->lockPath, _("transaction"));
 
-    if (rpmlockAcquire(ts->lock)) {
-	txn = xcalloc(1, sizeof(*txn));
+    int lockmode = (flags & RPMTXN_WRITE) ? RPMLOCK_WRITE : RPMLOCK_READ;
+    if (rpmlockAcquire(ts->lock, lockmode)) {
+	txn = (rpmtxn)xcalloc(1, sizeof(*txn));
 	txn->lock = ts->lock;
 	txn->flags = flags;
 	txn->ts = rpmtsLink(ts);
