@@ -10,12 +10,13 @@
 #include <rpm/rpmfi.h>
 #include <rpm/rpmstring.h>
 #include <rpm/rpmlog.h>
-#include "lib/misc.h"		/* tag function proto */
+#include <rpm/rpmbase64.h>
+#include "misc.h"		/* tag function proto */
 
 #include "debug.h"
 
 struct headerTagFunc_s {
-    rpmTag tag;		/*!< Tag of extension. */
+    rpmTagVal tag;		/*!< Tag of extension. */
     headerTagTagFunction func;	/*!< Pointer to formatter function. */	
 };
 
@@ -40,7 +41,7 @@ struct headerTagFunc_s {
 static int fnTag(Header h, rpmTag tagN, int withstate, rpmtd td)
 {
     const char **baseNames, **dirNames;
-    const char *fileStates = NULL;
+    const uint8_t *fileStates = NULL;
     uint32_t *dirIndexes;
     rpm_count_t count, retcount, dncount;
     size_t size = 0;
@@ -75,15 +76,15 @@ static int fnTag(Header h, rpmTag tagN, int withstate, rpmtd td)
 	    goto exit;
 	if (rpmtdCount(&fstates) != count)
 	    td->flags |= RPMTD_INVALID;
-	fileStates = fstates.data;
+	fileStates = (const uint8_t *)fstates.data;
     }
 
     if (td->flags & RPMTD_INVALID)
 	goto exit;
 
-    baseNames = bnames.data;
-    dirNames = dnames.data;
-    dirIndexes = dixs.data;
+    baseNames = (const char **)bnames.data;
+    dirNames = (const char **)dnames.data;
+    dirIndexes = (uint32_t *)dixs.data;
 
     /*
      * fsm, psm and rpmfi assume the data is stored in a single allocation
@@ -104,7 +105,7 @@ static int fnTag(Header h, rpmTag tagN, int withstate, rpmtd td)
     }
 
     if (!(td->flags & RPMTD_INVALID)) {
-	char **fileNames = xmalloc(size + (sizeof(*fileNames) * retcount));
+	char **fileNames = (char **)xmalloc(size + (sizeof(*fileNames) * retcount));
 	char *t = ((char *) fileNames) + (sizeof(*fileNames) * retcount);
 	for (i = 0, j = 0; i < count; i++) {
 	    if (fileStates && !RPMFILE_IS_INSTALLED(fileStates[i]))
@@ -147,13 +148,9 @@ static int filedepTag(Header h, rpmTag tagN, rpmtd td, headerGetFlags hgflags)
 	goto exit;
     }
 
-    if (tagN == RPMTAG_PROVIDENAME)
-	deptype = 'P';
-    else if (tagN == RPMTAG_REQUIRENAME)
-	deptype = 'R';
-
     ds = rpmdsNew(h, tagN, 0);
-    fdeps = xmalloc(numfiles * sizeof(*fdeps));
+    deptype = rpmdsD(ds);
+    fdeps = (char **)xmalloc(numfiles * sizeof(*fdeps));
 
     while ((fileix = rpmfiNext(fi)) >= 0) {
 	ARGV_t deps = NULL;
@@ -167,9 +164,7 @@ static int filedepTag(Header h, rpmTag tagN, rpmtd td, headerGetFlags hgflags)
 		if (mydt != deptype)
 		    continue;
 		dix &= 0x00ffffff;
-		(void) rpmdsSetIx(ds, dix-1);
-		if (rpmdsNext(ds) < 0)
-		    continue;
+		(void) rpmdsSetIx(ds, dix);
 		DNEVR = rpmdsDNEVR(ds);
 		if (DNEVR != NULL) {
 		    argvAdd(&deps, DNEVR + 2);
@@ -253,10 +248,10 @@ static int triggercondsTagFor(tMode mode, Header h, rpmtd td,
 
     td->type = RPM_STRING_ARRAY_TYPE;
     td->flags = RPMTD_ALLOCED | RPMTD_PTR_ALLOCED;
-    td->data = conds = xmalloc(sizeof(*conds) * rpmtdCount(&scripttd));
+    td->data = conds = (char **)xmalloc(sizeof(*conds) * rpmtdCount(&scripttd));
     td->count = rpmtdCount(&scripttd);
 
-    indices = indextd.data;
+    indices = (uint32_t *)indextd.data;
 
     while ((i = rpmtdNext(&scripttd)) >= 0) {
 	rpm_flag_t *flag;
@@ -358,7 +353,7 @@ static int triggertypeTagFor(tMode mode, Header h, rpmtd td,
 
     td->flags = RPMTD_ALLOCED | RPMTD_PTR_ALLOCED;
     td->count = rpmtdCount(&scripts);
-    td->data = conds = xmalloc(sizeof(*conds) * td->count);
+    td->data = conds = (char **)xmalloc(sizeof(*conds) * td->count);
     td->type = RPM_STRING_ARRAY_TYPE;
 
     while ((i = rpmtdNext(&scripts)) >= 0) {
@@ -493,7 +488,7 @@ static int fileclassTag(Header h, rpmtd td, headerGetFlags hgflags)
     int numfiles = rpmfiFC(fi);
 
     if (numfiles > 0) {
-	char **fclasses = xmalloc(numfiles * sizeof(*fclasses));
+	char **fclasses = (char **)xmalloc(numfiles * sizeof(*fclasses));
 	int ix;
 
 	rpmfiInit(fi, 0);
@@ -537,7 +532,7 @@ static int filerequireTag(Header h, rpmtd td, headerGetFlags hgflags)
 
 /* I18N look aside diversions */
 
-#if defined(ENABLE_NLS)
+#if defined(ENABLE_NLS) && defined(HAVE_NL_MSG_CAT_CNTR)
 extern int _nl_msg_cat_cntr;	/* XXX GNU gettext voodoo */
 #endif
 static const char * const language = "LANGUAGE";
@@ -574,7 +569,9 @@ static int i18nTag(Header h, rpmTag tag, rpmtd td, headerGetFlags hgflags)
 	/* change to en_US for msgkey -> msgid resolution */
 	langval = getenv(language);
 	(void) setenv(language, "en_US", 1);
+#if defined(HAVE_NL_MSG_CAT_CNTR)
         ++_nl_msg_cat_cntr;
+#endif
 
 	msgid = NULL;
 	for (domain = dstring; domain != NULL; domain = de) {
@@ -589,11 +586,13 @@ static int i18nTag(Header h, rpmTag tag, rpmtd td, headerGetFlags hgflags)
 	    (void) setenv(language, langval, 1);
 	else
 	    unsetenv(language);
+#if defined(HAVE_NL_MSG_CAT_CNTR)
         ++_nl_msg_cat_cntr;
+#endif
 
 	if (domain && msgid) {
 	    td->data = dgettext(domain, msgid);
-	    td->data = xstrdup(td->data); /* XXX xstrdup has side effects. */
+	    td->data = xstrdup((const char *)td->data); /* XXX xstrdup has side effects. */
 	    td->count = 1;
 	    td->flags = RPMTD_ALLOCED;
 	}
@@ -669,7 +668,7 @@ static int get64(Header h, rpmtd td, rpmTag newtag, rpmTag oldtag)
 	    td->count = olddata.count;
 	    td->flags = RPMTD_ALLOCED;
 	    td->data = xmalloc(sizeof(*d64) * td->count);
-	    d64 = td->data;
+	    d64 = (uint64_t *)td->data;
 	    while ((d32 = rpmtdNextUint32(&olddata))) {
 		*d64++ = *d32;
 	    }
@@ -710,7 +709,7 @@ static int longsigsizeTag(Header h, rpmtd td, headerGetFlags hgflags)
 
 static int numberTag(rpmtd td, uint32_t val)
 {
-    uint32_t *tval = xmalloc(sizeof(*tval));
+    uint32_t *tval = (uint32_t *)xmalloc(sizeof(*tval));
 
     tval[0] = val;
     td->type = RPM_INT32_TYPE;
@@ -855,7 +854,7 @@ static int epochnumTag(Header h, rpmtd td, headerGetFlags hgflags)
 {
     /* For consistency, always return malloced data */
     if (!headerGet(h, RPMTAG_EPOCH, td, HEADERGET_ALLOC)) {
-	uint32_t *e = malloc(sizeof(*e));
+	uint32_t *e = (uint32_t *)malloc(sizeof(*e));
 	*e = 0;
 	td->data = e;
 	td->type = RPM_INT32_TYPE;
@@ -873,7 +872,7 @@ static int depnevrsTag(Header h, rpmtd td, headerGetFlags hgflags,
     int ndeps = rpmdsCount(ds);
 
     if (ndeps > 0) {
-	char **deps = xmalloc(sizeof(*deps) * ndeps);
+	char **deps = (char **)xmalloc(sizeof(*deps) * ndeps);
 	int i;
 	while ((i = rpmdsNext(ds)) >= 0) {
 	    deps[i] = rpmdsNewDNEVR(NULL, ds);
@@ -896,7 +895,7 @@ static int depnevrsTagFiltered(Header h, rpmtd td, headerGetFlags hgflags,
     int ndeps = rpmdsCount(ds);
 
     if (ndeps > 0) {
-	char **deps = xmalloc(sizeof(*deps) * ndeps);
+	char **deps = (char **)xmalloc(sizeof(*deps) * ndeps);
 	ndeps = 0;
 	while (rpmdsNext(ds) >= 0) {
 	    if ((rpmdsFlags(ds) & RPMSENSE_STRONG) == (strong ? RPMSENSE_STRONG : 0))
@@ -965,7 +964,7 @@ static int filenlinksTag(Header h, rpmtd td, headerGetFlags hgflags)
     rpm_count_t fc = rpmfiFC(fi);
 
     if (fc > 0) {
-	uint32_t *nlinks = xmalloc(fc * sizeof(*nlinks));
+	uint32_t *nlinks = (uint32_t *)xmalloc(fc * sizeof(*nlinks));
 	int ix;
 	while ((ix = rpmfiNext(fi)) >= 0) {
 	    nlinks[ix] = rpmfiFNlink(fi);
@@ -980,6 +979,30 @@ static int filenlinksTag(Header h, rpmtd td, headerGetFlags hgflags)
     return (fc > 0);
 }
 
+static int sysusersTag(Header h, rpmtd td, headerGetFlags hgflags)
+{
+    ARGV_t sysusers = NULL;
+    rpmds provides = rpmdsNew(h, RPMTAG_PROVIDENAME, 0);
+
+    while (rpmdsNext(provides) >= 0) {
+	char *line = NULL;
+	if (rpmdsIsSysuser(provides, &line)) {
+	    argvAdd(&sysusers, line);
+	    free(line);
+	}
+    }
+    rpmdsFree(provides);
+
+    if (sysusers) {
+	td->count = argvCount(sysusers);
+	td->data = sysusers;
+	td->flags = RPMTD_ALLOCED|RPMTD_PTR_ALLOCED;
+	td->type = RPM_STRING_ARRAY_TYPE;
+    }
+
+    return (td->count > 0);
+}
+
 static const struct headerTagFunc_s rpmHeaderTagExtensions[] = {
     { RPMTAG_GROUP,		groupTag },
     { RPMTAG_DESCRIPTION,	descriptionTag },
@@ -1019,6 +1042,7 @@ static const struct headerTagFunc_s rpmHeaderTagExtensions[] = {
     { RPMTAG_OBSOLETENEVRS,	obsoletenevrsTag },
     { RPMTAG_CONFLICTNEVRS,	conflictnevrsTag },
     { RPMTAG_FILENLINKS,	filenlinksTag },
+    { RPMTAG_SYSUSERS,		sysusersTag },
     { 0, 			NULL }
 };
 
