@@ -10,10 +10,7 @@
 #include <rpm/rpmlog.h>
 #include <rpm/rpmstring.h>
 #include <rpm/rpmmacro.h>
-
-#include "lib/rpmfs.h"
-#include "lib/rpmplugin.h"
-#include "lib/rpmte_internal.h"
+#include <rpm/rpmplugin.h>
 
 #define XATTR_NAME_IMA "security.ima"
 
@@ -49,7 +46,7 @@ static rpmRC ima_fsm_file_prepare(rpmPlugin plugin, rpmfi fi, int fd,
 {
 	const unsigned char * fsig = NULL;
 	size_t len;
-	int rc = RPMRC_OK;
+	rpmRC rc = RPMRC_OK;
 	rpmFileAction action = XFO_ACTION(op);
 
 	/* Ignore skipped files and unowned directories */
@@ -74,7 +71,9 @@ static rpmRC ima_fsm_file_prepare(rpmPlugin plugin, rpmfi fi, int fd,
 	    else
 		xx = lsetxattr(path, XATTR_NAME_IMA, fsig, len, 0);
 	    if (xx < 0) {
-		int is_err = errno != EOPNOTSUPP;
+		/* unsupported fs or root inside rootless container? */
+		int is_err = !(errno == EOPNOTSUPP ||
+			      (errno == EPERM && getuid() == 0));
 
 	        rpmlog(is_err?RPMLOG_ERR:RPMLOG_DEBUG,
 			"ima: could not apply signature on '%s': %s\n",
