@@ -82,6 +82,12 @@ rpmds_IsReverse(rpmdsObject * s)
     return PyBool_FromLong(rpmdsIsReverse(s->ds));
 }
 
+static PyObject *
+rpmds_IsSysuser(rpmdsObject * s)
+{
+    return PyBool_FromLong(rpmdsIsSysuser(s->ds, NULL));
+}
+
 static PyObject *
 rpmds_iternext(rpmdsObject * s)
 {
@@ -102,20 +108,6 @@ rpmds_iternext(rpmdsObject * s)
     return result;
 }
 
-static PyObject *
-rpmds_SetNoPromote(rpmdsObject * s, PyObject * args, PyObject * kwds)
-{
-    int nopromote;
-    char * kwlist[] = {"noPromote", NULL};
-
-    DEPRECATED_METHOD("method is obsolete");
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "i:SetNoPromote", kwlist,
-	    &nopromote))
-	return NULL;
-
-    return Py_BuildValue("i", 0);
-}
-
 /* XXX rpmdsFind uses bsearch on s->ds, so a sort is needed. */
 static PyObject *
 rpmds_Sort(rpmdsObject * s)
@@ -129,7 +121,7 @@ rpmds_Find(rpmdsObject * s, PyObject * arg)
 {
     rpmdsObject * o;
 
-    if (!PyArg_Parse(arg, "O!:Find", &rpmds_Type, &o))
+    if (!PyArg_Parse(arg, "O!:Find", rpmds_Type, &o))
 	return NULL;
 
     /* XXX make sure ods index is valid, real fix in lib/rpmds.c. */
@@ -143,7 +135,7 @@ rpmds_Merge(rpmdsObject * s, PyObject * arg)
 {
     rpmdsObject * o;
 
-    if (!PyArg_Parse(arg, "O!:Merge", &rpmds_Type, &o))
+    if (!PyArg_Parse(arg, "O!:Merge", rpmds_Type, &o))
 	return NULL;
 
     return Py_BuildValue("i", rpmdsMerge(&s->ds, o->ds));
@@ -153,7 +145,7 @@ rpmds_Search(rpmdsObject * s, PyObject * arg)
 {
     rpmdsObject * o;
 
-    if (!PyArg_Parse(arg, "O!:Merge", &rpmds_Type, &o))
+    if (!PyArg_Parse(arg, "O!:Merge", rpmds_Type, &o))
         return NULL;
 
     return Py_BuildValue("i", rpmdsSearch(s->ds, o->ds));
@@ -163,7 +155,7 @@ static PyObject *rpmds_Compare(rpmdsObject * s, PyObject * o)
 {
     rpmdsObject * ods;
 
-    if (!PyArg_Parse(o, "O!:Compare", &rpmds_Type, &ods))
+    if (!PyArg_Parse(o, "O!:Compare", rpmds_Type, &ods))
 	return NULL;
 
     return PyBool_FromLong(rpmdsCompare(s->ds, ods->ds));
@@ -187,7 +179,7 @@ static PyObject * rpmds_Rpmlib(rpmdsObject * s, PyObject *args, PyObject *kwds)
     /* XXX check return code, permit arg (NULL uses system default). */
     rpmdsRpmlibPool(pool, &ds, NULL);
 
-    return rpmds_Wrap(&rpmds_Type, ds);
+    return rpmds_Wrap(rpmds_Type, ds);
 }
 
 static struct PyMethodDef rpmds_methods[] = {
@@ -208,8 +200,6 @@ static struct PyMethodDef rpmds_methods[] = {
   "the type of all dependencies in this set.\n" },
  {"Color",	(PyCFunction)rpmds_Color,	METH_NOARGS,
 	"ds.Color -> Color -- Return current Color.\n" },
- {"SetNoPromote",(PyCFunction)rpmds_SetNoPromote, METH_VARARGS|METH_KEYWORDS,
-  "ds.SetNoPromote(noPromote) -- Obsolete, do not use.\n"},
  {"Sort",	(PyCFunction)rpmds_Sort,	METH_NOARGS,
 	NULL},
  {"Find",	(PyCFunction)rpmds_Find,	METH_O,
@@ -232,6 +222,8 @@ The current index in ds is positioned at overlapping member." },
   "ds.IsRich() -- Return whether the dependency is rich."},
  {"IsReverse",	(PyCFunction)rpmds_IsReverse,	METH_NOARGS,
   "ds.IsReverse() -- Return whether the dependency is reversed."},
+ {"IsSysuser",	(PyCFunction)rpmds_IsSysuser,	METH_NOARGS,
+  "ds.IsSysuser() -- Return whether the dependency represents a sysusers.d entry."},
  {NULL,		NULL}		/* sentinel */
 };
 
@@ -241,7 +233,8 @@ static void
 rpmds_dealloc(rpmdsObject * s)
 {
     s->ds = rpmdsFree(s->ds);
-    Py_TYPE(s)->tp_free((PyObject *)s);
+    freefunc free = PyType_GetSlot(Py_TYPE(s), Py_tp_free);
+    free(s);
 }
 
 static Py_ssize_t rpmds_length(rpmdsObject * s)
@@ -264,11 +257,6 @@ rpmds_subscript(rpmdsObject * s, PyObject * key)
     return utf8FromString(rpmdsDNEVR(s->ds));
 }
 
-static PyMappingMethods rpmds_as_mapping = {
-        (lenfunc) rpmds_length,		/* mp_length */
-        (binaryfunc) rpmds_subscript,	/* mp_subscript */
-        (objobjargproc)0,		/* mp_ass_subscript */
-};
 
 static int rpmds_init(rpmdsObject * s, PyObject *args, PyObject *kwds)
 {
@@ -368,48 +356,27 @@ static char rpmds_doc[] =
     "a provide of the header NEVR."
     ;
 
-PyTypeObject rpmds_Type = {
-	PyVarObject_HEAD_INIT(&PyType_Type, 0)
-	"rpm.ds",			/* tp_name */
-	sizeof(rpmdsObject),		/* tp_basicsize */
-	0,				/* tp_itemsize */
-	/* methods */
-	(destructor) rpmds_dealloc,	/* tp_dealloc */
-	0,				/* tp_print */
-	(getattrfunc)0,			/* tp_getattr */
-	(setattrfunc)0,			/* tp_setattr */
-	0,				/* tp_compare */
-	(reprfunc)0,			/* tp_repr */
-	0,				/* tp_as_number */
-	0,				/* tp_as_sequence */
-	&rpmds_as_mapping,		/* tp_as_mapping */
-	(hashfunc)0,			/* tp_hash */
-	(ternaryfunc)0,			/* tp_call */
-	(reprfunc)0,			/* tp_str */
-	PyObject_GenericGetAttr,	/* tp_getattro */
-	PyObject_GenericSetAttr,	/* tp_setattro */
-	0,				/* tp_as_buffer */
-	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE, /* tp_flags */
-	rpmds_doc,			/* tp_doc */
-	0,				/* tp_traverse */
-	0,				/* tp_clear */
-	0,				/* tp_richcompare */
-	0,				/* tp_weaklistoffset */
-	PyObject_SelfIter,		/* tp_iter */
-	(iternextfunc) rpmds_iternext,	/* tp_iternext */
-	rpmds_methods,			/* tp_methods */
-	0,				/* tp_members */
-	0,				/* tp_getset */
-	0,				/* tp_base */
-	0,				/* tp_dict */
-	0,				/* tp_descr_get */
-	0,				/* tp_descr_set */
-	0,				/* tp_dictoffset */
-	(initproc) rpmds_init,		/* tp_init */
-	0,				/* tp_alloc */
-	(newfunc) rpmds_new,		/* tp_new */
-	0,				/* tp_free */
-	0,				/* tp_is_gc */
+static PyType_Slot rpmds_Type_Slots[] = {
+    {Py_tp_dealloc, rpmds_dealloc},
+    {Py_mp_length, rpmds_length},
+    {Py_mp_subscript, rpmds_subscript},
+    {Py_tp_getattro, PyObject_GenericGetAttr},
+    {Py_tp_setattro, PyObject_GenericSetAttr},
+    {Py_tp_doc, rpmds_doc},
+    {Py_tp_iter, PyObject_SelfIter},
+    {Py_tp_iternext, rpmds_iternext},
+    {Py_tp_methods, rpmds_methods},
+    {Py_tp_init, rpmds_init},
+    {Py_tp_new, rpmds_new},
+    {0, NULL},
+};
+
+PyTypeObject* rpmds_Type;
+PyType_Spec rpmds_Type_Spec = {
+    .name = "rpm.ds",
+    .basicsize = sizeof(rpmdsObject),
+    .flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_IMMUTABLETYPE,
+    .slots = rpmds_Type_Slots,
 };
 
 /* ---------- */
@@ -421,7 +388,8 @@ rpmds dsFromDs(rpmdsObject * s)
 
 PyObject * rpmds_Wrap(PyTypeObject *subtype, rpmds ds)
 {
-    rpmdsObject * s = (rpmdsObject *)subtype->tp_alloc(subtype, 0);
+    allocfunc subtype_alloc = (allocfunc)PyType_GetSlot(subtype, Py_tp_alloc);
+    rpmdsObject *s = (rpmdsObject *)subtype_alloc(subtype, 0);
     if (s == NULL) return NULL;
 
     s->ds = ds;
