@@ -76,7 +76,7 @@ rpmmi_iternext(rpmmiObject * s)
 	return NULL;
     }
     headerLink(h);
-    return hdr_Wrap(&hdr_Type, h);
+    return hdr_Wrap(hdr_Type, h);
 }
 
 static PyObject *
@@ -129,7 +129,8 @@ static void rpmmi_dealloc(rpmmiObject * s)
 {
     s->mi = rpmdbFreeIterator(s->mi);
     Py_DECREF(s->ref);
-    Py_TYPE(s)->tp_free((PyObject *)s);
+    freefunc free = PyType_GetSlot(Py_TYPE(s), Py_tp_free);
+    free(s);
 }
 
 static Py_ssize_t rpmmi_length(rpmmiObject * s)
@@ -142,23 +143,7 @@ static int rpmmi_bool(rpmmiObject *s)
     return (s->mi != NULL);
 }
 
-PyMappingMethods rpmmi_as_mapping = {
-    (lenfunc) rpmmi_length,		/* mp_length */
-    0,
-};
 
-static PyNumberMethods rpmmi_as_number = {
-	0, /* nb_add */
-	0, /* nb_subtract */
-	0, /* nb_multiply */
-	0, /* nb_remainder */
-	0, /* nb_divmod */
-	0, /* nb_power */
-	0, /* nb_negative */
-	0, /* nb_positive */
-	0, /* nb_absolute */
-	(inquiry)rpmmi_bool, /* nb_bool/nonzero */
-};
 
 static char rpmmi_doc[] =
   "rpm.mi match iterator object represents the result of a\n"
@@ -196,52 +181,40 @@ static char rpmmi_doc[] =
   "	    print('%s-%s-%s' % (h['name'], h['version'], h['release']))\n"
 ;
 
-PyTypeObject rpmmi_Type = {
-	PyVarObject_HEAD_INIT(&PyType_Type, 0)
-	"rpm.mi",			/* tp_name */
-	sizeof(rpmmiObject),		/* tp_size */
-	0,				/* tp_itemsize */
-	(destructor) rpmmi_dealloc, 	/* tp_dealloc */
-	0,				/* tp_print */
-	(getattrfunc)0, 		/* tp_getattr */
-	0,				/* tp_setattr */
-	0,				/* tp_compare */
-	0,				/* tp_repr */
-	&rpmmi_as_number,		/* tp_as_number */
-	0,				/* tp_as_sequence */
-	&rpmmi_as_mapping,		/* tp_as_mapping */
-	0,				/* tp_hash */
-	0,				/* tp_call */
-	0,				/* tp_str */
-	PyObject_GenericGetAttr,	/* tp_getattro */
-	PyObject_GenericSetAttr,	/* tp_setattro */
-	0,				/* tp_as_buffer */
-	Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE,	/* tp_flags */
-	rpmmi_doc,			/* tp_doc */
-	0,				/* tp_traverse */
-	0,				/* tp_clear */
-	0,				/* tp_richcompare */
-	0,				/* tp_weaklistoffset */
-	PyObject_SelfIter,		/* tp_iter */
-	(iternextfunc) rpmmi_iternext,	/* tp_iternext */
-	rpmmi_methods,			/* tp_methods */
-	0,				/* tp_members */
-	0,				/* tp_getset */
-	0,				/* tp_base */
-	0,				/* tp_dict */
-	0,				/* tp_descr_get */
-	0,				/* tp_descr_set */
-	0,				/* tp_dictoffset */
-	0,				/* tp_init */
-	0,				/* tp_alloc */
-	0,				/* tp_new */
-	0,				/* tp_free */
-	0,				/* tp_is_gc */
+static PyObject *disabled_new(PyTypeObject *type,
+                              PyObject *args, PyObject *kwds)
+{
+    PyErr_SetString(PyExc_TypeError,
+                    "TypeError: cannot create 'rpm.mi' instances");
+    return NULL;
+}
+
+static PyType_Slot rpmmi_Type_Slots[] = {
+    {Py_tp_new, disabled_new},
+    {Py_tp_dealloc, rpmmi_dealloc},
+    {Py_nb_bool, rpmmi_bool},
+    {Py_mp_length, rpmmi_length},
+    {Py_tp_getattro, PyObject_GenericGetAttr},
+    {Py_tp_setattro, PyObject_GenericSetAttr},
+    {Py_tp_doc, rpmmi_doc},
+    {Py_tp_iter, PyObject_SelfIter},
+    {Py_tp_iternext, rpmmi_iternext},
+    {Py_tp_methods, rpmmi_methods},
+    {0, NULL},
+};
+
+PyTypeObject* rpmmi_Type;
+PyType_Spec rpmmi_Type_Spec = {
+    .name = "rpm.mi",
+    .basicsize = sizeof(rpmmiObject),
+    .flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_IMMUTABLETYPE,
+    .slots = rpmmi_Type_Slots,
 };
 
 PyObject * rpmmi_Wrap(PyTypeObject *subtype, rpmdbMatchIterator mi, PyObject *s)
 {
-    rpmmiObject * mio = (rpmmiObject *)subtype->tp_alloc(subtype, 0);
+    allocfunc subtype_alloc = (allocfunc)PyType_GetSlot(subtype, Py_tp_alloc);
+    rpmmiObject *mio = (rpmmiObject *)subtype_alloc(subtype, 0);
     if (mio == NULL) return NULL;
 
     mio->mi = mi;
