@@ -67,56 +67,41 @@ static PyObject *rpmprob_str(rpmProblemObject *s)
 static void rpmprob_dealloc(rpmProblemObject *s)
 {
     s->prob = rpmProblemFree(s->prob);
-    Py_TYPE(s)->tp_free((PyObject *)s);
+    freefunc free = PyType_GetSlot(Py_TYPE(s), Py_tp_free);
+    free(s);
 }
 
-PyTypeObject rpmProblem_Type = {
-	PyVarObject_HEAD_INIT(&PyType_Type, 0)
-	"rpm.prob",			/* tp_name */
-	sizeof(rpmProblemObject),		/* tp_basicsize */
-	0,				/* tp_itemsize */
-	/* methods */
-	(destructor)rpmprob_dealloc,	/* tp_dealloc */
-	0,				/* tp_print */
-	(getattrfunc)0,			/* tp_getattr */
-	(setattrfunc)0,			/* tp_setattr */
-	0,				/* tp_compare */
-	(reprfunc)0,			/* tp_repr */
-	0,				/* tp_as_number */
-	0,				/* tp_as_sequence */
-	0,				/* tp_as_mapping */
-	(hashfunc)0,			/* tp_hash */
-	(ternaryfunc)0,			/* tp_call */
-	(reprfunc)rpmprob_str,		/* tp_str */
-	PyObject_GenericGetAttr,	/* tp_getattro */
-	PyObject_GenericSetAttr,	/* tp_setattro */
-	0,				/* tp_as_buffer */
-	Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE,	/* tp_flags */
-	rpmprob_doc,			/* tp_doc */
-	0,				/* tp_traverse */
-	0,				/* tp_clear */
-	(richcmpfunc)0,			/* tp_richcompare */
-	0,				/* tp_weaklistoffset */
-	0,				/* tp_iter */
-	0,				/* tp_iternext */
-	0,				/* tp_methods */
-	0,				/* tp_members */
-	rpmprob_getseters,		/* tp_getset */
-	0,				/* tp_base */
-	0,				/* tp_dict */
-	0,				/* tp_descr_get */
-	0,				/* tp_descr_set */
-	0,				/* tp_dictoffset */
-	0,				/* tp_init */
-	0,				/* tp_alloc */
-	(newfunc)0,			/* tp_new */
-	0,				/* tp_free */
-	0,				/* tp_is_gc */
+static PyObject *disabled_new(PyTypeObject *type,
+                              PyObject *args, PyObject *kwds)
+{
+    PyErr_SetString(PyExc_TypeError,
+                    "TypeError: cannot create 'rpm.prob' instances");
+    return NULL;
+}
+
+static PyType_Slot rpmProblem_Type_Slots[] = {
+    {Py_tp_new, disabled_new},
+    {Py_tp_dealloc, rpmprob_dealloc},
+    {Py_tp_str, rpmprob_str},
+    {Py_tp_getattro, PyObject_GenericGetAttr},
+    {Py_tp_setattro, PyObject_GenericSetAttr},
+    {Py_tp_doc, rpmprob_doc},
+    {Py_tp_getset, rpmprob_getseters},
+    {0, NULL},
+};
+
+PyTypeObject* rpmProblem_Type;
+PyType_Spec rpmProblem_Type_Spec = {
+    .name = "rpm.prob",
+    .basicsize = sizeof(rpmProblemObject),
+    .flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_IMMUTABLETYPE,
+    .slots = rpmProblem_Type_Slots,
 };
 
 PyObject *rpmprob_Wrap(PyTypeObject *subtype, rpmProblem prob)
 {
-    rpmProblemObject * s = (rpmProblemObject *)subtype->tp_alloc(subtype, 0);
+    allocfunc subtype_alloc = (allocfunc)PyType_GetSlot(subtype, Py_tp_alloc);
+    rpmProblemObject *s = (rpmProblemObject *)subtype_alloc(subtype, 0);
     if (s == NULL) return NULL;
 
     s->prob = rpmProblemLink(prob);
@@ -137,7 +122,7 @@ PyObject *rpmps_AsList(rpmps ps)
     psi = rpmpsInitIterator(ps);
 
     while ((prob = rpmpsiNext(psi))) {
-        PyObject *pyprob = rpmprob_Wrap(&rpmProblem_Type, prob);
+        PyObject *pyprob = rpmprob_Wrap(rpmProblem_Type, prob);
         if (!pyprob) {
             Py_DECREF(problems);
             rpmpsFreeIterator(psi);
