@@ -2,7 +2,6 @@
 
 #include "header-py.h"	/* XXX tagNumFromPyObject */
 #include "rpmds-py.h"
-#include "rpmfi-py.h"
 #include "rpmfiles-py.h"
 #include "rpmte-py.h"
 #include "rpmps-py.h"
@@ -30,15 +29,14 @@
  * - te.PkgFileSize() Return no. of bytes in package file (approx).
  * - te.Parent() Return the parent element index.
  * - te.Problems() Return problems associated with this element.
- * - te.AddedKey() Return the added package index (TR_ADDED).
- * - te.DependsOnKey() Return the package index for the added package (TR_REMOVED).
- * - te.DBOffset() Return the Packages database instance number (TR_REMOVED)
+ * - te.DBOffset() Return the Packages database instance number
  * - te.Key()	Return the associated opaque key, i.e. 2nd arg ts.addInstall().
  * - te.DS(tag)	Return package dependency set.
  * @param tag	'Providename', 'Requirename', 'Obsoletename', 'Conflictname'
  * - te.FI(tag)	Return package file info set iterator (deprecated).
  * @param tag	'Basenames'
  * - te.Files()	Return package file info set.
+ * - te.Verified() Return package verification status.
  */
 
 struct rpmteObject_s {
@@ -56,49 +54,49 @@ rpmte_TEType(rpmteObject * s, PyObject * unused)
 static PyObject *
 rpmte_N(rpmteObject * s, PyObject * unused)
 {
-    return Py_BuildValue("s", rpmteN(s->te));
+    return utf8FromString(rpmteN(s->te));
 }
 
 static PyObject *
 rpmte_E(rpmteObject * s, PyObject * unused)
 {
-    return Py_BuildValue("s", rpmteE(s->te));
+    return utf8FromString(rpmteE(s->te));
 }
 
 static PyObject *
 rpmte_V(rpmteObject * s, PyObject * unused)
 {
-    return Py_BuildValue("s", rpmteV(s->te));
+    return utf8FromString(rpmteV(s->te));
 }
 
 static PyObject *
 rpmte_R(rpmteObject * s, PyObject * unused)
 {
-    return Py_BuildValue("s", rpmteR(s->te));
+    return utf8FromString(rpmteR(s->te));
 }
 
 static PyObject *
 rpmte_A(rpmteObject * s, PyObject * unused)
 {
-    return Py_BuildValue("s", rpmteA(s->te));
+    return utf8FromString(rpmteA(s->te));
 }
 
 static PyObject *
 rpmte_O(rpmteObject * s, PyObject * unused)
 {
-    return Py_BuildValue("s", rpmteO(s->te));
+    return utf8FromString(rpmteO(s->te));
 }
 
 static PyObject *
 rpmte_NEVR(rpmteObject * s, PyObject * unused)
 {
-    return Py_BuildValue("s", rpmteNEVR(s->te));
+    return utf8FromString(rpmteNEVR(s->te));
 }
 
 static PyObject *
 rpmte_NEVRA(rpmteObject * s, PyObject * unused)
 {
-    return Py_BuildValue("s", rpmteNEVRA(s->te));
+    return utf8FromString(rpmteNEVRA(s->te));
 }
 
 static PyObject *
@@ -118,7 +116,7 @@ rpmte_Parent(rpmteObject * s, PyObject * unused)
 {
     rpmte parent = rpmteParent(s->te);
     if (parent)
-	return rpmte_Wrap(&rpmte_Type, parent);
+	return rpmte_Wrap(rpmte_Type, parent);
 
     Py_RETURN_NONE;
 }
@@ -136,14 +134,6 @@ static PyObject * rpmte_Problems(rpmteObject * s, PyObject * unused)
     return problems;
 }
 
-/*
-static PyObject *
-rpmte_DependsOnKey(rpmteObject * s, PyObject * unused)
-{
-    return Py_BuildValue("i", rpmteDependsOnKey(s->te));
-}
-*/
-
 static PyObject *
 rpmte_DBOffset(rpmteObject * s, PyObject * unused)
 {
@@ -161,6 +151,30 @@ rpmte_Key(rpmteObject * s, PyObject * unused)
     return Key;
 }
 
+static PyObject *
+rpmte_SetUserdata(rpmteObject * s, PyObject *arg)
+{
+    /* XXX how to insure this is a PyObject??? */
+    PyObject *o = rpmteUserdata(s->te);
+    rpmteSetUserdata(s->te, arg);
+    Py_INCREF(arg);
+    Py_XDECREF(o);
+    Py_RETURN_NONE;
+}
+
+static PyObject *
+rpmte_Userdata(rpmteObject * s)
+{
+    PyObject *po = Py_None;
+    void *data = rpmteUserdata(s->te);
+
+    if (data)
+	po = data;
+
+    Py_INCREF(po);
+    return po;
+}
+
 static PyObject *
 rpmte_DS(rpmteObject * s, PyObject * args, PyObject * kwds)
 {
@@ -176,21 +190,7 @@ rpmte_DS(rpmteObject * s, PyObject * args, PyObject * kwds)
     if (ds == NULL) {
 	Py_RETURN_NONE;
     }
-    return rpmds_Wrap(&rpmds_Type, rpmdsLink(ds));
-}
-
-static PyObject *
-rpmte_FI(rpmteObject * s, PyObject * args, PyObject * kwds)
-{
-    rpmfi fi;
-
-    DEPRECATED_METHOD("use .Files() instead");
-
-    fi = rpmteFI(s->te);
-    if (fi == NULL) {
-	Py_RETURN_NONE;
-    }
-    return rpmfi_Wrap(&rpmfi_Type, rpmfiLink(fi));
+    return rpmds_Wrap(rpmds_Type, rpmdsLink(ds));
 }
 
 static PyObject *
@@ -200,8 +200,16 @@ rpmte_Files(rpmteObject * s, PyObject * args, PyObject * kwds)
     if (files == NULL) {
 	Py_RETURN_NONE;
     }
-    return rpmfiles_Wrap(&rpmfiles_Type, files);
+    return rpmfiles_Wrap(rpmfiles_Type, files);
 }
+
+static PyObject *
+rpmte_Verified(rpmteObject * s)
+{
+    return Py_BuildValue("i", rpmteVerified(s->te));
+}
+
+
 static struct PyMethodDef rpmte_methods[] = {
     {"Type",	(PyCFunction)rpmte_TEType,	METH_NOARGS,
      "te.Type() -- Return element type (rpm.TR_ADDED | rpm.TR_REMOVED).\n" },
@@ -229,24 +237,26 @@ static struct PyMethodDef rpmte_methods[] = {
      "te.Parent() -- Return the parent element index."},
     {"Problems",(PyCFunction)rpmte_Problems,	METH_NOARGS,
      "te.Problems() -- Return problems associated with this element."},
-/*    {"DependsOnKey",(PyCFunction)rpmte_DependsOnKey,	METH_NOARGS,
-      NULL}, */
     {"DBOffset",(PyCFunction)rpmte_DBOffset,	METH_NOARGS,
-     "te.DBOffset() -- Return the Package's database instance number.\n\nTR_REMOVED only"},
+     "te.DBOffset() -- Return the Package's database instance number.\n"},
     {"Failed",	(PyCFunction)rpmte_Failed,	METH_NOARGS,
      "te.Failed() -- Return if there are any related errors."},
     {"Key",	(PyCFunction)rpmte_Key,		METH_NOARGS,
-     "te.Key() -- Return the associated opaque key aka user data\n\
+     "te.Key() -- Return the associated opaque retrieval key\n\
 	as passed e.g. as data arg ts.addInstall()"},
+    {"Userdata",	(PyCFunction)rpmte_Userdata,	METH_NOARGS,
+     "te.Userdata() -- Return associated user data (if any)\n"},
+    {"SetUserdata",	(PyCFunction)rpmte_SetUserdata,	METH_O,
+     "te.SetUserdata() -- Set associated user data (if any)\n"},
     {"DS",	(PyCFunction)rpmte_DS,		METH_VARARGS|METH_KEYWORDS,
 "te.DS(TagN) -- Return the TagN dependency set (or None).\n\
 	TagN is one of 'Providename', 'Requirename', 'Obsoletename',\n\
 	'Conflictname', 'Triggername', 'Recommendname', 'Suggestname',\n\
 	'Supplementname', 'Enhancename'" },
-    {"FI",	(PyCFunction)rpmte_FI,		METH_VARARGS|METH_KEYWORDS,
-"te.FI(TagN) -- Return file info iterator of element.\n\n DEPRECATED! Use .Files() instead.\n" },
     {"Files",	(PyCFunction)rpmte_Files,	METH_NOARGS,
 "te.Files() -- Return file info set of element.\n" },
+    {"Verified",(PyCFunction)rpmte_Verified,	METH_NOARGS,
+"te.Verified() -- Return element verification status.\n" },
     {NULL,		NULL}		/* sentinel */
 };
 
@@ -255,52 +265,35 @@ static struct PyMethodDef rpmte_methods[] = {
 static char rpmte_doc[] =
 "";
 
-PyTypeObject rpmte_Type = {
-	PyVarObject_HEAD_INIT(&PyType_Type, 0)
-	"rpm.te",			/* tp_name */
-	sizeof(rpmteObject),		/* tp_size */
-	0,				/* tp_itemsize */
-	(destructor)0,		 	/* tp_dealloc */
-	0,				/* tp_print */
-	(getattrfunc)0,		 	/* tp_getattr */
-	(setattrfunc)0,			/* tp_setattr */
-	0,				/* tp_compare */
-	0,				/* tp_repr */
-	0,				/* tp_as_number */
-	0,				/* tp_as_sequence */
-	0,				/* tp_as_mapping */
-	0,				/* tp_hash */
-	0,				/* tp_call */
-	0,				/* tp_str */
-	PyObject_GenericGetAttr,	/* tp_getattro */
-	PyObject_GenericSetAttr,	/* tp_setattro */
-	0,				/* tp_as_buffer */
-	Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE,	/* tp_flags */
-	rpmte_doc,			/* tp_doc */
-	0,				/* tp_traverse */
-	0,				/* tp_clear */
-	0,				/* tp_richcompare */
-	0,				/* tp_weaklistoffset */
-	0,				/* tp_iter */
-	0,				/* tp_iternext */
-	rpmte_methods,			/* tp_methods */
-	0,				/* tp_members */
-	0,				/* tp_getset */
-	0,				/* tp_base */
-	0,				/* tp_dict */
-	0,				/* tp_descr_get */
-	0,				/* tp_descr_set */
-	0,				/* tp_dictoffset */
-	0,				/* tp_init */
-	0,				/* tp_alloc */
-	0,				/* tp_new */
-	0,				/* tp_free */
-	0,				/* tp_is_gc */
+static PyObject *disabled_new(PyTypeObject *type,
+                              PyObject *args, PyObject *kwds)
+{
+    PyErr_SetString(PyExc_TypeError,
+                    "TypeError: cannot create 'rpm.te' instances");
+    return NULL;
+}
+
+static PyType_Slot rpmte_Type_Slots[] = {
+    {Py_tp_new, disabled_new},
+    {Py_tp_getattro, PyObject_GenericGetAttr},
+    {Py_tp_setattro, PyObject_GenericSetAttr},
+    {Py_tp_doc, rpmte_doc},
+    {Py_tp_methods, rpmte_methods},
+    {0, NULL},
+};
+
+PyTypeObject* rpmte_Type;
+PyType_Spec rpmte_Type_Spec = {
+    .name = "rpm.te",
+    .basicsize = sizeof(rpmteObject),
+    .flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_IMMUTABLETYPE,
+    .slots = rpmte_Type_Slots,
 };
 
 PyObject * rpmte_Wrap(PyTypeObject *subtype, rpmte te)
 {
-    rpmteObject *s = (rpmteObject *)subtype->tp_alloc(subtype, 0);
+    allocfunc subtype_alloc = (allocfunc)PyType_GetSlot(subtype, Py_tp_alloc);
+    rpmteObject *s = (rpmteObject *)subtype_alloc(subtype, 0);
     if (s == NULL) return NULL;
 
     s->te = te;
