@@ -16,10 +16,6 @@ typedef struct rpmhookArgs_s {
 
 typedef int (*rpmhookFunc) (rpmhookArgs args, void *data);
 
-#ifdef __cplusplus
-extern "C" {
-#endif
- 
 rpmhookArgs rpmhookArgsNew(int argc);
 rpmhookArgs rpmhookArgsFree(rpmhookArgs args);
 
@@ -30,8 +26,4 @@ void rpmhookUnregisterAll(const char *name);
 void rpmhookCall(const char *name, const char *argt, ...);
 void rpmhookCallArgs(const char *name, rpmhookArgs args);
 
-#ifdef __cplusplus
-}
-#endif
-
 #endif
