@@ -8,10 +8,6 @@
 #include <ctype.h>
 #include <dirent.h>
 #include <fcntl.h>
-#if defined(__linux__)
-#include <sys/personality.h>
-#endif
-#include <sys/utsname.h>
 #include <sys/resource.h>
 
 #include <rpm/rpmlog.h>
@@ -20,7 +16,7 @@
 #include <rpm/rpmsw.h>
 #include <rpm/rpmurl.h>
 
-#include "rpmio/rpmio_internal.h"
+#include "rpmio_internal.h"
 
 #include "debug.h"
 
@@ -80,7 +76,7 @@ static void fdSetFdno(FD_t fd, int fdno)
 
 static void fdPush(FD_t fd, FDIO_t io, void * fp, int fdno)
 {
-    FDSTACK_t fps = xcalloc(1, sizeof(*fps));
+    FDSTACK_t fps = (FDSTACK_t) xcalloc(1, sizeof(*fps));
     fps->io = io;
     fps->fp = fp;
     fps->fdno = fdno;
@@ -147,21 +143,6 @@ struct FDIO_s {
   fdio_fstrerr_function_t	_fstrerr;
 };
 
-/* forward refs */
-static const FDIO_t fdio;
-static const FDIO_t ufdio;
-static const FDIO_t gzdio;
-#if HAVE_BZLIB_H
-static const FDIO_t bzdio;
-#endif
-#ifdef HAVE_LZMA_H
-static const FDIO_t xzdio;
-static const FDIO_t lzdio;
-#endif
-#ifdef HAVE_ZSTD
-static const FDIO_t zstdio;
-#endif
-
 /** \ingroup rpmio
  * Update digest(s) attached to fd.
  */
@@ -354,12 +335,12 @@ FD_t fdFree( FD_t fd)
 
 static FD_t fdNew(int fdno, const char *descr)
 {
-    FD_t fd = xcalloc(1, sizeof(*fd));
+    FD_t fd = (FD_t)xcalloc(1, sizeof(*fd));
     fd->nrefs = 0;
     fd->flags = 0;
     fd->magic = FDMAGIC;
     fd->urlType = URL_IS_UNKNOWN;
-    fd->stats = xcalloc(1, sizeof(*fd->stats));
+    fd->stats = (FDSTAT_t)xcalloc(1, sizeof(*fd->stats));
     fd->digests = NULL;
     fd->descr = descr ? xstrdup(descr) : NULL;
 
@@ -423,7 +404,7 @@ static const struct FDIO_s fdio_s = {
   fdRead, fdWrite, fdSeek, fdClose,
   fdOpen, NULL, fdFlush, fdTell, fdError, fdStrerr,
 };
-static const FDIO_t fdio = &fdio_s ;
+const FDIO_t fdio = &fdio_s ;
 
 off_t ufdCopy(FD_t sfd, FD_t tfd)
 {
@@ -477,6 +458,7 @@ static FD_t urlOpen(const char * url, int flags, mode_t mode)
 	unlink(dest);
     } else {
 	fd = NULL;
+	errno = ENOENT;
     }
     dest = _free(dest);
 
@@ -523,24 +505,20 @@ fprintf(stderr, "*** ufdOpen(%s,0x%x,0%o)\n", url, (unsigned)flags, (unsigned)mo
     return fd;
 }
 
-/* Return number of threads ought to be used for compression based
-   on a parsed value threads (e.g. from w7T0.xzdio or w7T16.xzdio).
-   Value -1 means automatic detection. */
-
-static int
-get_compression_threads(int threads)
-{
-    if (threads == -1)
-	threads = rpmExpandNumeric("%{getncpus}");
-
-#if __WORDSIZE == 32
-    /* Limit threads up to 4 for 32-bit systems.  */
-    if (threads > 4) {
-	threads = 4;
-	rpmlog(RPMLOG_DEBUG, "threading compression limited to 4 threads on 32-bit systems\n");
-    }
+#if !ENABLE_OPENMP
+#define omp_in_parallel() 0
 #endif
 
+/* Parse thread enablement string. T or T0 means autodetection */
+static int parsethreadn(const char *s, char **end)
+{
+    int threads = strtol(s, end, 10);
+    if (threads == 0) {
+	/* Disable autodetection inside OpenMP parallel regions */
+	if (!omp_in_parallel())
+	    threads = rpmExpandNumeric("%{getncpus:thread}");
+    }
+
     return threads;
 }
 
@@ -549,11 +527,15 @@ static const struct FDIO_s ufdio_s = {
   fdRead, fdWrite, fdSeek, fdClose,
   ufdOpen, NULL, fdFlush, fdTell, fdError, fdStrerr
 };
-static const FDIO_t ufdio = &ufdio_s ;
+const FDIO_t ufdio = &ufdio_s ;
 
 /* =============================================================== */
 /* Support for GZIP library.  */
 #include <zlib.h>
+static gzFile gzdFp(FDSTACK_t fps)
+{
+    return (gzFile)fps->fp;
+}
 
 static FD_t gzdFdopen(FD_t fd, int fdno, const char *fmode)
 {
@@ -569,14 +551,14 @@ static FD_t gzdFdopen(FD_t fd, int fdno, const char *fmode)
 
 static int gzdFlush(FDSTACK_t fps)
 {
-    gzFile gzfile = fps->fp;
+    gzFile gzfile = gzdFp(fps);
     if (gzfile == NULL) return -2;
     return gzflush(gzfile, Z_SYNC_FLUSH);	/* XXX W2DO? */
 }
 
 static void gzdSetError(FDSTACK_t fps)
 {
-    gzFile gzfile = fps->fp;
+    gzFile gzfile = gzdFp(fps);
     int zerror = 0;
     fps->errcookie = gzerror(gzfile, &zerror);
     if (zerror == Z_ERRNO) {
@@ -587,7 +569,7 @@ static void gzdSetError(FDSTACK_t fps)
 
 static ssize_t gzdRead(FDSTACK_t fps, void * buf, size_t count)
 {
-    gzFile gzfile = fps->fp;
+    gzFile gzfile = gzdFp(fps);
     ssize_t rc;
 
     if (gzfile == NULL) return -2;	/* XXX can't happen */
@@ -603,7 +585,7 @@ static ssize_t gzdWrite(FDSTACK_t fps, const void * buf, size_t count)
     gzFile gzfile;
     ssize_t rc;
 
-    gzfile = fps->fp;
+    gzfile = gzdFp(fps);
     if (gzfile == NULL) return -2;	/* XXX can't happen */
 
     rc = gzwrite(gzfile, (void *)buf, count);
@@ -612,28 +594,23 @@ static ssize_t gzdWrite(FDSTACK_t fps, const void * buf, size_t count)
     return rc;
 }
 
-/* XXX zlib-1.0.4 has not */
 static int gzdSeek(FDSTACK_t fps, off_t pos, int whence)
 {
     off_t p = pos;
     int rc;
-#if HAVE_GZSEEK
-    gzFile gzfile = fps->fp;
+    gzFile gzfile = gzdFp(fps);
 
     if (gzfile == NULL) return -2;	/* XXX can't happen */
 
     rc = gzseek(gzfile, p, whence);
     if (rc < 0)
 	gzdSetError(fps);
-#else
-    rc = -2;
-#endif
     return rc;
 }
 
 static int gzdClose(FDSTACK_t fps)
 {
-    gzFile gzfile = fps->fp;
+    gzFile gzfile = gzdFp(fps);
     int rc;
 
     if (gzfile == NULL) return -2;	/* XXX can't happen */
@@ -646,10 +623,10 @@ static int gzdClose(FDSTACK_t fps)
 static off_t gzdTell(FDSTACK_t fps)
 {
     off_t pos = -1;
-    gzFile gzfile = fps->fp;
+    gzFile gzfile = gzdFp(fps);
 
     if (gzfile != NULL) {
-#if HAVE_GZSEEK
+#ifdef HAVE_GZSEEK
 	pos = gztell(gzfile);
 	if (pos < 0)
 	    gzdSetError(fps);
@@ -664,14 +641,19 @@ static const struct FDIO_s gzdio_s = {
   gzdRead, gzdWrite, gzdSeek, gzdClose,
   NULL, gzdFdopen, gzdFlush, gzdTell, zfdError, zfdStrerr
 };
-static const FDIO_t gzdio = &gzdio_s ;
+const FDIO_t gzdio = &gzdio_s ;
 
 /* =============================================================== */
 /* Support for BZIP2 library.  */
-#if HAVE_BZLIB_H
+#ifdef HAVE_BZLIB_H
 
 #include <bzlib.h>
 
+static BZFILE *bzdFp(FDSTACK_t fps)
+{
+    return (BZFILE *)fps->fp;
+}
+
 static FD_t bzdFdopen(FD_t fd, int fdno, const char * fmode)
 {
     BZFILE *bzfile = BZ2_bzdopen(fdno, fmode);
@@ -686,12 +668,12 @@ static FD_t bzdFdopen(FD_t fd, int fdno, const char * fmode)
 
 static int bzdFlush(FDSTACK_t fps)
 {
-    return BZ2_bzflush(fps->fp);
+    return BZ2_bzflush(bzdFp(fps));
 }
 
 static ssize_t bzdRead(FDSTACK_t fps, void * buf, size_t count)
 {
-    BZFILE *bzfile = fps->fp;
+    BZFILE *bzfile = bzdFp(fps);
     ssize_t rc = 0;
 
     if (bzfile)
@@ -707,7 +689,7 @@ static ssize_t bzdRead(FDSTACK_t fps, void * buf, size_t count)
 
 static ssize_t bzdWrite(FDSTACK_t fps, const void * buf, size_t count)
 {
-    BZFILE *bzfile = fps->fp;
+    BZFILE *bzfile = bzdFp(fps);
     ssize_t rc;
 
     rc = BZ2_bzwrite(bzfile, (void *)buf, count);
@@ -720,7 +702,7 @@ static ssize_t bzdWrite(FDSTACK_t fps, const void * buf, size_t count)
 
 static int bzdClose(FDSTACK_t fps)
 {
-    BZFILE *bzfile = fps->fp;
+    BZFILE *bzfile = bzdFp(fps);
 
     if (bzfile == NULL) return -2;
 
@@ -735,7 +717,7 @@ static const struct FDIO_s bzdio_s = {
   bzdRead, bzdWrite, NULL, bzdClose,
   NULL, bzdFdopen, bzdFlush, NULL, zfdError, zfdStrerr
 };
-static const FDIO_t bzdio = &bzdio_s ;
+const FDIO_t bzdio = &bzdio_s ;
 
 #endif	/* HAVE_BZLIB_H */
 
@@ -747,10 +729,6 @@ static const FDIO_t bzdio = &bzdio_s ;
 #include <sys/types.h>
 #include <inttypes.h>
 #include <lzma.h>
-/* Multithreading support in stable API since xz 5.2.0 */
-#if LZMA_VERSION >= 50020002
-#define HAVE_LZMA_MT
-#endif
 
 #define kBufferSize (1 << 15)
 
@@ -769,16 +747,15 @@ typedef struct lzfile {
 
 static LZFILE *lzopen_internal(const char *mode, int fd, int xz)
 {
-    int level = LZMA_PRESET_DEFAULT;
+    unsigned int level = LZMA_PRESET_DEFAULT;
     int encoding = 0;
     FILE *fp;
     LZFILE *lzfile;
     lzma_ret ret;
     lzma_stream init_strm = LZMA_STREAM_INIT;
     uint64_t mem_limit = rpmExpandNumeric("%{_xz_memlimit}");
-#ifdef HAVE_LZMA_MT
-    int threads = 0;
-#endif
+    unsigned int threads = 0;
+
     for (; *mode; mode++) {
 	if (*mode == 'w')
 	    encoding = 1;
@@ -787,42 +764,24 @@ static LZFILE *lzopen_internal(const char *mode, int fd, int xz)
 	else if (*mode >= '0' && *mode <= '9')
 	    level = *mode - '0';
 	else if (*mode == 'T') {
-	    if (isdigit(*(mode+1))) {
-#ifdef HAVE_LZMA_MT
-		threads = atoi(++mode);
-		/* T0 means automatic detection */
-		if (threads == 0)
-		    threads = -1;
-#endif
-		/* skip past rest of digits in string that atoi()
-		 * should've processed
-		 * */
-		while (isdigit(*++mode));
-		--mode;
-	    }
-#ifdef HAVE_LZMA_MT
-	    else
-		threads = -1;
-#endif
+	    char *end = NULL;
+	    threads = parsethreadn(mode+1, &end);
+	    mode = end-1;
 	}
     }
     fp = fdopen(fd, encoding ? "w" : "r");
     if (!fp)
 	return NULL;
-    lzfile = calloc(1, sizeof(*lzfile));
+    lzfile = (LZFILE *)calloc(1, sizeof(*lzfile));
     lzfile->file = fp;
     lzfile->encoding = encoding;
     lzfile->eof = 0;
     lzfile->strm = init_strm;
     if (encoding) {
 	if (xz) {
-#ifdef HAVE_LZMA_MT
 	    if (!threads) {
-#endif
 		ret = lzma_easy_encoder(&lzfile->strm, level, LZMA_CHECK_SHA256);
-#ifdef HAVE_LZMA_MT
 	    } else {
-		threads = get_compression_threads(threads);
 		lzma_mt mt_options = {
 		    .flags = 0,
 		    .threads = threads,
@@ -834,7 +793,6 @@ static LZFILE *lzopen_internal(const char *mode, int fd, int xz)
 
 		ret = lzma_stream_encoder_mt(&lzfile->strm, &mt_options);
 	    }
-#endif
 	} else {
 	    lzma_options_lzma options;
 	    lzma_lzma_preset(&options, level);
@@ -901,7 +859,7 @@ static ssize_t lzread(LZFILE *lzfile, void *buf, size_t len)
       return -1;
     if (lzfile->eof)
       return 0;
-    lzfile->strm.next_out = buf;
+    lzfile->strm.next_out = (uint8_t *)buf;
     lzfile->strm.avail_out = len;
     for (;;) {
 	if (!lzfile->strm.avail_in) {
@@ -932,7 +890,7 @@ static ssize_t lzwrite(LZFILE *lzfile, void *buf, size_t len)
 	return -1;
     if (!len)
 	return 0;
-    lzfile->strm.next_in = buf;
+    lzfile->strm.next_in = (uint8_t *)buf;
     lzfile->strm.avail_in = len;
     for (;;) {
 	lzfile->strm.next_out = lzfile->buf;
@@ -960,6 +918,11 @@ static FD_t xzdFdopen(FD_t fd, int fdno, const char * fmode)
     return fd;
 }
 
+static LZFILE *lzdFp(FDSTACK_t fps)
+{
+    return (LZFILE*)fps->fp;
+}
+
 static FD_t lzdFdopen(FD_t fd, int fdno, const char * fmode)
 {
     LZFILE *lzfile = lzopen_internal(fmode, fdno, 0);
@@ -974,13 +937,13 @@ static FD_t lzdFdopen(FD_t fd, int fdno, const char * fmode)
 
 static int lzdFlush(FDSTACK_t fps)
 {
-    LZFILE *lzfile = fps->fp;
+    LZFILE *lzfile = lzdFp(fps);
     return fflush(lzfile->file);
 }
 
 static ssize_t lzdRead(FDSTACK_t fps, void * buf, size_t count)
 {
-    LZFILE *lzfile = fps->fp;
+    LZFILE *lzfile = lzdFp(fps);
     ssize_t rc = 0;
 
     if (lzfile)
@@ -993,7 +956,7 @@ static ssize_t lzdRead(FDSTACK_t fps, void * buf, size_t count)
 
 static ssize_t lzdWrite(FDSTACK_t fps, const void * buf, size_t count)
 {
-    LZFILE *lzfile = fps->fp;
+    LZFILE *lzfile = lzdFp(fps);
     ssize_t rc = 0;
 
     rc = lzwrite(lzfile, (void *)buf, count);
@@ -1005,7 +968,7 @@ static ssize_t lzdWrite(FDSTACK_t fps, const void * buf, size_t count)
 
 static int lzdClose(FDSTACK_t fps)
 {
-    LZFILE *lzfile = fps->fp;
+    LZFILE *lzfile = lzdFp(fps);
     int rc;
 
     if (lzfile == NULL) return -2;
@@ -1019,14 +982,14 @@ static struct FDIO_s xzdio_s = {
   lzdRead, lzdWrite, NULL, lzdClose,
   NULL, xzdFdopen, lzdFlush, NULL, zfdError, zfdStrerr
 };
-static const FDIO_t xzdio = &xzdio_s;
+const FDIO_t xzdio = &xzdio_s;
 
 static struct FDIO_s lzdio_s = {
   "lzdio", "lzma",
   lzdRead, lzdWrite, NULL, lzdClose,
   NULL, lzdFdopen, lzdFlush, NULL, zfdError, zfdStrerr
 };
-static const FDIO_t lzdio = &lzdio_s;
+const FDIO_t lzdio = &lzdio_s;
 
 #endif	/* HAVE_LZMA_H */
 
@@ -1041,7 +1004,10 @@ typedef struct rpmzstd_s {
     int fdno;
     int level;			/*!< compression level */
     FILE * fp;
-    void * _stream;             /*!< ZSTD_{C,D}Stream */
+    union {
+	ZSTD_DStream *d;
+	ZSTD_CStream *c;
+    } stream;
     size_t nb;
     void * b;
     ZSTD_inBuffer zib;          /*!< ZSTD_inBuffer */
@@ -1050,6 +1016,7 @@ typedef struct rpmzstd_s {
 
 static rpmzstd rpmzstdNew(int fdno, const char *fmode)
 {
+    rpmzstd zstd = NULL;
     int flags = 0;
     int level = 3;
     const char * s = fmode;
@@ -1089,11 +1056,7 @@ static rpmzstd rpmzstdNew(int fdno, const char *fmode)
 	    flags |= O_RDWR;
 	    continue;
 	case 'T':
-	    if (*s >= '0' && *s <= '9')
-		threads = strtol(s, (char **)&s, 10);
-	    /* T0 means automatic detection */
-	    if (threads == 0)
-		threads = -1;
+	    threads = parsethreadn(s, (char **)&s);
 	    continue;
     case 'L':
 	    c = *s++;
@@ -1134,57 +1097,59 @@ static rpmzstd rpmzstdNew(int fdno, const char *fmode)
     if (fp == NULL)
 	return NULL;
 
-    void * _stream = NULL;
+    zstd = (rpmzstd) xcalloc(1, sizeof(*zstd));
     size_t nb = 0;
 
     if ((flags & O_ACCMODE) == O_RDONLY) {	/* decompressing */
-	if ((_stream = (void *) ZSTD_createDStream()) == NULL
-	 || ZSTD_isError(ZSTD_initDStream(_stream))) {
+	if ((zstd->stream.d = ZSTD_createDStream()) == NULL
+	 || ZSTD_isError(ZSTD_initDStream(zstd->stream.d))) {
 	    goto err;
 	}
 	nb = ZSTD_DStreamInSize();
     } else {					/* compressing */
-	if ((_stream = (void *) ZSTD_createCCtx()) == NULL
-	 || ZSTD_isError(ZSTD_CCtx_setParameter(_stream, ZSTD_c_compressionLevel, level))) {
+	if ((zstd->stream.c = ZSTD_createCCtx()) == NULL
+	 || ZSTD_isError(ZSTD_CCtx_setParameter(zstd->stream.c, ZSTD_c_compressionLevel, level))) {
 	    goto err;
 	}
 
 	if (longdist) {
-	    if (ZSTD_isError(ZSTD_CCtx_setParameter(_stream, ZSTD_c_enableLongDistanceMatching, longdist))
-	     || ZSTD_isError(ZSTD_CCtx_setParameter(_stream, ZSTD_c_windowLog, windowlog))) {
+	    if (ZSTD_isError(ZSTD_CCtx_setParameter(zstd->stream.c, ZSTD_c_enableLongDistanceMatching, longdist))
+	     || ZSTD_isError(ZSTD_CCtx_setParameter(zstd->stream.c, ZSTD_c_windowLog, windowlog))) {
 	        goto err;
 	    }
 	}
 
-	threads = get_compression_threads(threads);
 	if (threads > 0) {
-	    if (ZSTD_isError (ZSTD_CCtx_setParameter(_stream, ZSTD_c_nbWorkers, threads)))
+	    if (ZSTD_isError (ZSTD_CCtx_setParameter(zstd->stream.c, ZSTD_c_nbWorkers, threads)))
 		rpmlog(RPMLOG_DEBUG, "zstd library does not support multi-threading\n");
 	}
 
 	nb = ZSTD_CStreamOutSize();
     }
 
-    rpmzstd zstd = (rpmzstd) xcalloc(1, sizeof(*zstd));
     zstd->flags = flags;
     zstd->fdno = fdno;
     zstd->level = level;
     zstd->fp = fp;
-    zstd->_stream = _stream;
     zstd->nb = nb;
-    zstd->b = xmalloc(nb);
+    zstd->b = (uint8_t *)xmalloc(nb);
 
     return zstd;
 
 err:
     fclose(fp);
     if ((flags & O_ACCMODE) == O_RDONLY)
-	ZSTD_freeDStream(_stream);
+	ZSTD_freeDStream(zstd->stream.d);
     else
-	ZSTD_freeCCtx(_stream);
+	ZSTD_freeCCtx(zstd->stream.c);
     return NULL;
 }
 
+static rpmzstd zstdFp(FDSTACK_t fps)
+{
+    return (rpmzstd)fps->fp;
+}
+
 static FD_t zstdFdopen(FD_t fd, int fdno, const char * fmode)
 {
     rpmzstd zstd = rpmzstdNew(fdno, fmode);
@@ -1199,7 +1164,7 @@ static FD_t zstdFdopen(FD_t fd, int fdno, const char * fmode)
 
 static int zstdFlush(FDSTACK_t fps)
 {
-    rpmzstd zstd = (rpmzstd) fps->fp;
+    rpmzstd zstd = zstdFp(fps);
 assert(zstd);
     int rc = -1;
 
@@ -1213,7 +1178,7 @@ assert(zstd);
 	  zstd->zob.dst  = zstd->b;
 	  zstd->zob.size = zstd->nb;
 	  zstd->zob.pos  = 0;
-	  xx = ZSTD_compressStream2(zstd->_stream, &zstd->zob, &zib, ZSTD_e_flush);
+	  xx = ZSTD_compressStream2(zstd->stream.c, &zstd->zob, &zib, ZSTD_e_flush);
 	  if (ZSTD_isError(xx)) {
 	      fps->errcookie = ZSTD_getErrorName(xx);
 	      break;
@@ -1231,7 +1196,7 @@ assert(zstd);
 
 static ssize_t zstdRead(FDSTACK_t fps, void * buf, size_t count)
 {
-    rpmzstd zstd = (rpmzstd) fps->fp;
+    rpmzstd zstd = zstdFp(fps);
 assert(zstd);
     ZSTD_outBuffer zob = { buf, count, 0 };
 
@@ -1246,7 +1211,7 @@ assert(zstd);
 	}
 
 	/* Decompress next chunk. */
-	int xx = ZSTD_decompressStream(zstd->_stream, &zob, &zstd->zib);
+	int xx = ZSTD_decompressStream(zstd->stream.d, &zob, &zstd->zib);
 	if (ZSTD_isError(xx)) {
 	    fps->errcookie = ZSTD_getErrorName(xx);
 	    return -1;
@@ -1257,7 +1222,7 @@ assert(zstd);
 
 static ssize_t zstdWrite(FDSTACK_t fps, const void * buf, size_t count)
 {
-    rpmzstd zstd = (rpmzstd) fps->fp;
+    rpmzstd zstd = zstdFp(fps);
 assert(zstd);
     ZSTD_inBuffer zib = { buf, count, 0 };
 
@@ -1269,7 +1234,7 @@ assert(zstd);
 	zstd->zob.pos  = 0;
 
 	/* Compress next chunk. */
-	int xx = ZSTD_compressStream2(zstd->_stream, &zstd->zob, &zib, ZSTD_e_continue);
+	int xx = ZSTD_compressStream2(zstd->stream.c, &zstd->zob, &zib, ZSTD_e_continue);
 	if (ZSTD_isError(xx)) {
 	    fps->errcookie = ZSTD_getErrorName(xx);
 	    return -1;
@@ -1289,13 +1254,13 @@ assert(zstd);
 
 static int zstdClose(FDSTACK_t fps)
 {
-    rpmzstd zstd = (rpmzstd) fps->fp;
+    rpmzstd zstd = zstdFp(fps);
 assert(zstd);
     int rc = -2;
 
     if ((zstd->flags & O_ACCMODE) == O_RDONLY) { /* decompressing */
 	rc = 0;
-	ZSTD_freeDStream(zstd->_stream);
+	ZSTD_freeDStream(zstd->stream.d);
     } else {					/* compressing */
 	/* close frame */
 	int xx;
@@ -1304,7 +1269,7 @@ assert(zstd);
 	  zstd->zob.dst  = zstd->b;
 	  zstd->zob.size = zstd->nb;
 	  zstd->zob.pos  = 0;
-	  xx = ZSTD_compressStream2(zstd->_stream, &zstd->zob, &zib, ZSTD_e_end);
+	  xx = ZSTD_compressStream2(zstd->stream.c, &zstd->zob, &zib, ZSTD_e_end);
 	  if (ZSTD_isError(xx)) {
 	      fps->errcookie = ZSTD_getErrorName(xx);
 	      break;
@@ -1316,7 +1281,7 @@ assert(zstd);
 	  else
 	      rc = 0;
 	} while (xx != 0);
-	ZSTD_freeCCtx(zstd->_stream);
+	ZSTD_freeCCtx(zstd->stream.c);
     }
 
     if (zstd->fp && fileno(zstd->fp) > 2)
@@ -1333,7 +1298,7 @@ static const struct FDIO_s zstdio_s = {
   zstdRead, zstdWrite, NULL, zstdClose,
   NULL, zstdFdopen, zstdFlush, NULL, zfdError, zfdStrerr
 };
-static const FDIO_t zstdio = &zstdio_s ;
+const FDIO_t zstdio = &zstdio_s ;
 
 #endif	/* HAVE_ZSTD */
 
@@ -1546,10 +1511,10 @@ static FDIO_t findIOT(const char *name)
 	&fdio_s,
 	&ufdio_s,
 	&gzdio_s,
-#if HAVE_BZLIB_H
+#ifdef HAVE_BZLIB_H
 	&bzdio_s,
 #endif
-#if HAVE_LZMA_H
+#ifdef HAVE_LZMA_H
 	&xzdio_s,
 	&lzdio_s,
 #endif
@@ -1736,7 +1701,7 @@ int rpmioSlurp(const char * fn, uint8_t ** bp, ssize_t * blenp)
     blen = (size >= 0 ? size : blenmax);
     if (blen) {
 	int nb;
-	b = xmalloc(blen+1);
+	b = (uint8_t *)xmalloc(blen+1);
 	b[0] = '\0';
 	nb = Fread(b, sizeof(*b), blen, fd);
 	if (Ferror(fd) || (size > 0 && nb != blen)) {
