@@ -1,10 +1,5 @@
 #include "system.h"
 
-#include <lua.h>
-#include <lualib.h>
-#include <lauxlib.h>
-#include "lposix.h"
-
 #ifndef LUA_LOADED_TABLE
 /* feature introduced in Lua 5.3.4 */
 #define LUA_LOADED_TABLE "_LOADED"
@@ -26,11 +21,12 @@
 #include <rpm/rpmfileutil.h>
 #include <rpm/rpmbase64.h>
 #include <rpm/rpmver.h>
-#include "rpmio/rpmhook.h"
+#include "rpmhook.h"
 
-#include "rpmio/rpmlua.h"
-#include "rpmio/rpmio_internal.h"
-#include "rpmio/rpmmacro_internal.h"
+#include "rpmlua.h"
+#include "rpmio_internal.h"
+#include "rpmmacro_internal.h"
+#include "lposix.h"
 
 #include "debug.h"
 
@@ -185,7 +181,7 @@ void * rpmluaGetLua(rpmlua lua)
 void rpmluaPushPrintBuffer(rpmlua lua)
 {
     INITSTATE(lua);
-    rpmluapb prbuf = xcalloc(1, sizeof(*prbuf));
+    rpmluapb prbuf = (rpmluapb)xcalloc(1, sizeof(*prbuf));
     prbuf->buf = NULL;
     prbuf->alloced = 0;
     prbuf->used = 0;
@@ -228,8 +224,8 @@ int rpmluaCheckScript(rpmlua lua, const char *script, const char *name)
 
 static int luaopt(int c, const char *oarg, int oint, void *data)
 {
-    lua_State *L = data;
-    char key[2] = { c, '\0' };
+    lua_State *L = (lua_State *)data;
+    char key[2] = { (char)c, '\0' };
 
     lua_pushstring(L, oarg ? oarg : "");
     lua_setfield(L, -2, key);
@@ -239,17 +235,13 @@ static int luaopt(int c, const char *oarg, int oint, void *data)
 static int rpm_pcall(lua_State *L, int nargs, int nresults, int errfunc)
 {
     pid_t pid = getpid();
-    int status;
 
     int rc = lua_pcall(L, nargs, nresults, errfunc);
 
     /* Terminate unhandled fork from Lua script */
-    if (pid != getpid())
-	_exit(1);
-
-    if (waitpid(0, &status, WNOHANG) == 0) {
-	rpmlog(RPMLOG_WARNING,
-		_("runaway fork() in Lua script\n"));
+    if (pid != getpid()) {
+	rpmlog(RPMLOG_WARNING, _("runaway fork() in Lua script\n"));
+	_exit(EXIT_FAILURE);
     }
 
     return rc;
@@ -262,6 +254,7 @@ int rpmluaRunScript(rpmlua lua, const char *script, const char *name,
     lua_State *L = lua->L;
     int ret = -1;
     int oind = 0;
+    int nret = 0;
     static const char *lualocal = "local opt, arg = ...;";
     int otop = lua_gettop(L); /* this can recurse through macros */
 
@@ -309,7 +302,7 @@ int rpmluaRunScript(rpmlua lua, const char *script, const char *name,
 	goto exit;
     }
 
-    int nret = lua_gettop(L) - otop;
+    nret = lua_gettop(L) - otop;
     if (nret > 0 && lua->printbuf) {
 	lua_getglobal(L, "print");
 	lua_insert(L, -(nret + 1));
@@ -325,6 +318,8 @@ int rpmluaRunScript(rpmlua lua, const char *script, const char *name,
 
 exit:
     free(buf);
+    /* discard any unhandled return data from the script */
+    lua_settop(L, otop);
     return ret;
 }
 
@@ -347,7 +342,7 @@ int rpmluaRunScriptFile(rpmlua lua, const char *filename)
     return ret;
 }
 
-static char *lamereadline(char *prompt)
+static char *lamereadline(const char *prompt)
 {
     static char buffer[1024];
     if (prompt) {
@@ -361,7 +356,7 @@ static char *lamereadline(char *prompt)
 }
 
 /* From lua.c */
-static int rpmluaReadline(lua_State *L, char *prompt, rpmluarl rlcb)
+static int rpmluaReadline(lua_State *L, const char *prompt, rpmluarl rlcb)
 {
     char *line = rlcb(prompt);
     if (line) {
@@ -534,7 +529,7 @@ static int rpm_b64decode(lua_State *L)
 	void *data = NULL;
 	size_t len = 0;
 	if (rpmBase64Decode(str, &data, &len) == 0) {
-	    lua_pushlstring(L, data, len);
+	    lua_pushlstring(L, (char *)data, len);
 	} else {
 	    lua_pushnil(L);
 	}
@@ -696,7 +691,7 @@ static int rpm_register(lua_State *L)
 	(void) luaL_argerror(L, 2, "function expected");
     } else {
 	rpmluaHookData hookdata =
-	    lua_newuserdata(L, sizeof(struct rpmluaHookData_s));
+	    (rpmluaHookData)lua_newuserdata(L, sizeof(struct rpmluaHookData_s));
 	lua_pushvalue(L, -1);
 	hookdata->dataRef = luaL_ref(L, LUA_REGISTRYINDEX);
 	lua_pushvalue(L, 2);
@@ -792,6 +787,8 @@ static int rpm_redirect2null(lua_State *L)
 {
     int target_fd, fd, r, e;
 
+    check_deprecated(L, "rpm.redirect2null", "4.20.0");
+
     if (!_rpmlua_have_forked)
 	return luaL_error(L, "redirect2null not permitted in this context");
 
@@ -818,6 +815,83 @@ static int rpm_exit(lua_State *L)
     exit(luaL_optinteger(L, 1, EXIT_SUCCESS));
 }
 
+static int do_spawnop(lua_State *L, posix_spawn_file_actions_t *fap)
+{
+    const char *key = luaL_checkstring(L, -2);
+    const char *val = luaL_checkstring(L, -1);
+    int rc = 0;
+
+    if (rstreq(key, "stdin")) {
+	rc = posix_spawn_file_actions_addopen(fap, STDIN_FILENO,
+		    val, O_RDONLY, 0644);
+    } else if (rstreq(key, "stdout")) {
+	rc = posix_spawn_file_actions_addopen(fap, STDOUT_FILENO,
+		    val, O_WRONLY|O_APPEND|O_CREAT, 0644);
+    } else if (rstreq(key, "stderr")) {
+	rc = posix_spawn_file_actions_addopen(fap, STDERR_FILENO,
+		    val, O_WRONLY|O_APPEND|O_CREAT, 0644);
+    } else {
+	luaL_error(L, "invalid spawn directive: %s", key);
+    }
+
+    return rc;
+}
+
+static int rpm_spawn(lua_State *L)
+{
+    int rc = 0, status;
+    pid_t pid;
+    posix_spawn_file_actions_t fa, *fap = NULL;
+    luaL_checktype(L, 1, LUA_TTABLE);
+    int argc = luaL_len(L, 1);
+
+    if (argc == 0)
+	luaL_error(L, "command not supplied");
+
+    if (lua_istable(L, 2)) {
+	if ((rc = posix_spawn_file_actions_init(&fa)))
+	    return pusherror(L, rc, "spawn file action init");
+	fap = &fa;
+
+	lua_pushnil(L);
+	while (!rc && lua_next(L, 2) != 0) {
+	    rc = do_spawnop(L, &fa);
+	    lua_pop(L, 1);
+	};
+	if (rc)
+	    return pusherror(L, rc, NULL);
+    }
+
+    ARGV_t argv = NULL;
+    for (int i = 0; i < argc; i++) {
+	lua_rawgeti(L, 1, i + 1);
+	argvAdd(&argv, luaL_checkstring(L, -1));
+	lua_pop(L, 1);
+    }
+
+    rpmSetCloseOnExec();
+
+    status = posix_spawnp(&pid, argv[0], fap, NULL, argv, environ);
+
+    argvFree(argv);
+    if (fap)
+	posix_spawn_file_actions_destroy(fap);
+
+    if (status != 0)
+	return pusherror(L, status, NULL);
+    if (waitpid(pid, &status, 0) == -1)
+	return pusherror(L, errno, NULL);
+    if (status != 0) {
+	if (WIFSIGNALED(status)) {
+	    return pusherror(L, WTERMSIG(status), "exit signal");
+	} else {
+	    return pusherror(L, WEXITSTATUS(status), "exit code");
+	}
+    }
+
+    return pushresult(L, WEXITSTATUS(status));
+}
+
 static int rpm_execute(lua_State *L)
 {
     const char *file = luaL_checkstring(L, 1);
@@ -825,7 +899,7 @@ static int rpm_execute(lua_State *L)
     int status;
     pid_t pid;
 
-    char **argv = malloc((n + 1) * sizeof(char *));
+    char **argv = (char **)malloc((n + 1) * sizeof(char *));
     if (argv == NULL)
 	return luaL_error(L, "not enough memory");
     argv[0] = (char *)file;
@@ -839,8 +913,13 @@ static int rpm_execute(lua_State *L)
 	return pusherror(L, status, NULL);
     if (waitpid(pid, &status, 0) == -1)
 	return pusherror(L, errno, NULL);
-    if (status != 0)
-	return pusherror(L, status, "exit code");
+    if (status != 0) {
+	if (WIFSIGNALED(status)) {
+	    return pusherror(L, WTERMSIG(status), "exit signal");
+	} else {
+	    return pusherror(L, WEXITSTATUS(status), "exit code");
+	}
+    }
 
     return pushresult(L, status);
 }
@@ -857,6 +936,7 @@ static int rpm_splitargs(lua_State *L)
 	lua_pushstring(L, args[i]);
 	lua_rawseti(L, -2, i + 1);
     }
+    argvFree(args);
     return 1;
 }
 
@@ -878,14 +958,39 @@ static int rpm_unsplitargs(lua_State *L)
     str = args ? unsplitQuoted(args, " ") : NULL;
     lua_pushstring(L, str ? str : "");
     free(str);
+    argvFree(args);
+    return 1;
+}
+
+static int rpm_glob(lua_State *L)
+{
+    const char *pat = luaL_checkstring(L, 1);
+    rpmglobFlags flags = RPMGLOB_NONE;
+    int argc = 0;
+    ARGV_t argv = NULL;
+
+    if (luaL_optstring(L, 2, "c"))
+	flags |= RPMGLOB_NOCHECK;
+
+    if (rpmGlobPath(pat, flags, &argc, &argv) == 0) {
+	lua_createtable(L, 0, argc);
+	for (int i = 0; i < argc; i++) {
+	    lua_pushstring(L, argv[i]);
+	    lua_rawseti(L, -2, i + 1);
+	}
+	argvFree(argv);
+    } else {
+	luaL_error(L, "glob %s failed: %s", pat, strerror(errno));
+    }
+
     return 1;
 }
 
 static int newinstance(lua_State *L, const char *name, void *p)
 {
     if (p != NULL) {
-	intptr_t **pp = lua_newuserdata(L, sizeof(*pp));
-	*pp = p;
+	intptr_t **pp = (intptr_t **)lua_newuserdata(L, sizeof(*pp));
+	*pp = (intptr_t *)p;
 	luaL_getmetatable(L, name);
 	lua_setmetatable(L, -2);
     }
@@ -903,7 +1008,7 @@ static int createclass(lua_State *L, const char *name, const luaL_Reg *methods)
 
 static rpmver * checkver(lua_State *L, int ix)
 {
-    rpmver *vp = lua_touserdata(L, ix);
+    rpmver *vp = (rpmver *)lua_touserdata(L, ix);
     luaL_checkudata(L, ix, "rpm.ver");
     return vp;
 }
@@ -997,7 +1102,7 @@ static const luaL_Reg ver_m[] = {
 
 static FD_t * checkfd(lua_State *L, int ix)
 {
-    FD_t *fdp = lua_touserdata(L, ix);
+    FD_t *fdp = (FD_t *)lua_touserdata(L, ix);
     luaL_checkudata(L, ix, "rpm.fd");
     return fdp;
 }
@@ -1149,7 +1254,7 @@ static const luaL_Reg fd_m[] = {
 
 static rpmMacroContext *checkmc(lua_State *L, int ix)
 {
-    rpmMacroContext *mc = lua_touserdata(L, ix);
+    rpmMacroContext *mc = (rpmMacroContext *)lua_touserdata(L, ix);
     luaL_checkudata(L, ix, "rpm.mc");
     return mc;
 }
@@ -1181,6 +1286,7 @@ static int mc_call(lua_State *L)
 		argvAdd(&argv, s);
 		lua_pop(L, 1);
 	    } else {
+		argvFree(argv);
 		luaL_argerror(L, i, "cannot convert to string");
 	    }
 	}
@@ -1190,6 +1296,7 @@ static int mc_call(lua_State *L)
 	    lua_pushstring(L, buf);
 	    free(buf);
 	} else {
+	    argvFree(argv);
 	    luaL_error(L, "error expanding macro");
 	}
 	argvFree(argv);
@@ -1264,6 +1371,7 @@ static const luaL_Reg rpmlib[] = {
     {"unregister", rpm_unregister},
     {"call", rpm_call},
     {"interactive", rpm_interactive},
+    {"spawn", rpm_spawn},
     {"execute", rpm_execute},
     {"redirect2null", rpm_redirect2null},
     {"vercmp", rpm_vercmp},
@@ -1271,6 +1379,7 @@ static const luaL_Reg rpmlib[] = {
     {"open", rpm_open},
     {"splitargs", rpm_splitargs},
     {"unsplitargs", rpm_unsplitargs},
+    {"glob", rpm_glob},
     {NULL, NULL}
 };
 
@@ -1285,3 +1394,23 @@ static int luaopen_rpm(lua_State *L)
     luaL_newlib(L, rpmlib);
     return 1;
 }
+
+void check_deprecated(lua_State *L, const char *func, const char *deprecated_in)
+{
+    int warn = 1;
+    lua_getfield(L, LUA_REGISTRYINDEX, "RPM_PACKAGE_RPMVERSION");
+    if (lua_isstring(L, -1)) {
+	rpmver v1 = rpmverParse(lua_tostring(L, -1));
+	rpmver v2 = rpmverParse(deprecated_in);
+	if (v1 && v2 && rpmverCmp(v1, v2) < 0)
+	    warn = 0;
+	rpmverFree(v2);
+	rpmverFree(v1);
+    }
+    lua_pop(L, 1);
+
+    if (warn) {
+	fprintf(stderr,
+	    "warning: %s(): .fork(), .exec(), .wait() and .redirect2null() are deprecated, use rpm.spawn() or rpm.execute() instead\n", func);
+    }
+}
