@@ -16,6 +16,7 @@ int soname_only = 0;
 int fake_soname = 1;
 int filter_soname = 1;
 int require_interp = 0;
+int multifile = 0;
 
 typedef struct elfInfo_s {
     Elf *elf;
@@ -195,6 +196,8 @@ static void processVerNeed(Elf_Scn *scn, GElf_Shdr *shdr, elfInfo *ei)
 static void processDynamic(Elf_Scn *scn, GElf_Shdr *shdr, elfInfo *ei)
 {
     Elf_Data *data = NULL;
+    if (shdr->sh_entsize == 0)
+	return;
     while ((data = elf_getdata(scn, data)) != NULL) {
 	for (int i = 0; i < (shdr->sh_size / shdr->sh_entsize); i++) {
 	    const char *s = NULL;
@@ -280,7 +283,8 @@ static int processFile(const char *fn, int dtype)
     int fdno;
     struct stat st;
     GElf_Ehdr *ehdr, ehdr_mem;
-    elfInfo *ei = rcalloc(1, sizeof(*ei));
+    elfInfo *ei = (elfInfo *)rcalloc(1, sizeof(*ei));
+    ARGV_t dep = NULL;
 
     fdno = open(fn, O_RDONLY);
     if (fdno < 0 || fstat(fdno, &st) < 0)
@@ -332,8 +336,12 @@ static int processFile(const char *fn, int dtype)
 
     rc = 0;
     /* dump the requested dependencies for this file */
-    for (ARGV_t dep = dtype ? ei->requires : ei->provides; dep && *dep; dep++) {
-	fprintf(stdout, "%s\n", *dep);
+    dep = dtype ? ei->requires : ei->provides;
+    if (dep && *dep) {
+	if (multifile)
+	    fprintf(stdout, ";%s\n", fn);
+	for (; dep && *dep; dep++)
+	    fprintf(stdout, "%s\n", *dep);
     }
 
 exit:
@@ -363,6 +371,7 @@ int main(int argc, char *argv[])
 	{ "no-fake-soname", 0, POPT_ARG_VAL, &fake_soname, 0, NULL, NULL },
 	{ "no-filter-soname", 0, POPT_ARG_VAL, &filter_soname, 0, NULL, NULL },
 	{ "require-interp", 0, POPT_ARG_VAL, &require_interp, -1, NULL, NULL },
+	{ "multifile", 'm', POPT_ARG_VAL, &multifile, -1, NULL, NULL },
 	POPT_AUTOHELP 
 	POPT_TABLEEND
     };
